# -*- coding: utf-8 -*-
"""
method to open .mat files (Ries)
"""
from __future__ import print_function

import numpy as np

# On the windows machine the matlab binary import raised a warning.
# We want to catch that warning, since importing ries's files works.
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    try:
        # scipy.io might not work on OSX (wrong architecture)
        from scipy.io.matlab.mio5_params import mat_struct
        from scipy.io import loadmat
    except:
        print(" Error: import error in scipys 'matlab' submodule.")
        print("        Try upgrading python-scipy or ignore this")
        print("        error if you are not using .mat files that")
        print("        were generated by programs by Jonas Ries.")
import os


def openMAT(dirname, filename):
    """
        Read mat files that Jonas Ries used in his programs.
        For opening .mat files, this helped a lot:
        http://stackoverflow.com/questions/7008608/
        scipy-io-loadmat-nested-structures-i-e-dictionaries

        The structure has been derived from "corrSFCS.m" from the SFCS.m
        program from Jonas Ries.
    """
    # initiate lists
    correlations = list()
    traces = list()
    curvelist = list()
    # Import everything inside the mat file as big iterated dictionary
    f = os.path.join(dirname, filename)
    alldata = loadmat(f)
    # Correlation functions are stored in "g"
    g = alldata["g"]
    # Get all Autocorrelation functions
    try:
        # ac for autocorrelation
        ac = g["ac"]
    except KeyError:
        pass
    else:
        # Workaround for single ACs, they are not stored in a separate list,
        # but directly inserted into g["ac"]. We put it in a list.
        # This is not the case for the trace averages.
        # There are a maximum of 4 autocorrelation functions in one file,
        # as far as I know.
        if len(ac) > 4:
            ac = [ac]
            g["act"] = [g["act"]]
        for i in np.arange(len(ac)):
            corr = ac[i]
            try:
                times = g["act"][i]
            except KeyError:
                pass
            else:
                # Another workaround
                # Sometimes, there's just one curve, which
                # means that corr[0] has no length.
                if len( np.atleast_1d(corr[0]) ) == 1:
                    final = np.zeros((len(corr), 2))
                    final[:,0] = times
                    final[:,1] = corr
                    correlations.append(final)
                    curvelist.append("AC"+str(i+1))
                    try:
                        # only trace averages are saved
                        traceavg = g["trace"][i]
                    except:
                        # No trace
                        traces.append(None)
                    else:
                        trace = np.zeros((2,2))
                        trace[1,0] = 1.0
                        trace[:,1] = traceavg
                        traces.append(trace)


                elif len(corr) == len(times):
                    for j in np.arange(len(corr[0])):

                        final = np.zeros((len(corr), 2))
                        final[:,0] = times
                        final[:,1] = corr[:,j]
                        correlations.append(final)
                        curvelist.append("AC"+str(i+1))
                        try:
                            # only trace averages are saved
                            traceavg = g["trace"][i][j]
                        except:
                            # No trace
                            traces.append(None)
                        else:
                            trace = np.zeros((2,2))
                            trace[1,0] = 1.0
                            trace[:,1] = traceavg
                            traces.append(trace)
    # Get dc "dual color" functions
    try:
        dc = g["dc"]
    except KeyError:
        pass
    else:
        for i in np.arange(len(dc)):
            corr = dc[i]
            try:
                times = g["dct"][i]
            except KeyError:
                pass
            else:
                if len(corr) == len(times):
                    for j in np.arange(len(corr[0])):

                        final = np.zeros((len(corr), 2))
                        final[:,0] = times
                        final[:,1] = corr[:,j]
                        correlations.append(final)
                        curvelist.append("CC dual color "+str(i+1))
                        traces.append(None)
    # Get twof "two focus" functions
    try:
        twof = g["twof"]
    except KeyError:
        pass
    else:
        for i in np.arange(len(dc)):
            corr = twof[i]
            try:
                times = g["twoft"][i]
            except KeyError:
                pass
            else:
                if len(corr) == len(times):
                    for j in np.arange(len(corr[0])):

                        final = np.zeros((len(corr), 2))
                        final[:,0] = times
                        final[:,1] = corr[:,j]
                        correlations.append(final)
                        curvelist.append("CC two foci "+str(i+1))
                        traces.append(None)
    # Get dc2f "dual color two focus" functions
    try:
        g["dc2f"]
    except KeyError:
        pass
    else:
        for i in np.arange(len(dc)):
            corr = twof[i]
            try:
                times = g["dc2ft"][i]
            except KeyError:
                pass
            else:
                if len(corr) == len(times):
                    for j in np.arange(len(corr[0])):

                        final = np.zeros((len(corr), 2))
                        final[:,0] = times
                        final[:,1] = corr[:,j]
                        correlations.append(final)
                        curvelist.append("CC dual color two foci "+str(i+1))
                        traces.append(None)
    dictionary = dict()
    dictionary["Correlation"] = correlations
    dictionary["Trace"] = traces
    dictionary["Type"] = curvelist
    filelist = list()
    for i in curvelist:
        filelist.append(filename)
    dictionary["Filename"] = filelist
    return dictionary


def loadmat(filename):
    '''
    this function should be called instead of direct scipy.io.loadmat
    as it cures the problem of not properly recovering python dictionaries
    from mat files. It calls the function check keys to cure all entries
    which are still mat-objects
    '''
    data = loadmat(filename, struct_as_record=False, squeeze_me=True)
    return _check_keys(data)


def _check_keys(adict):
    '''
    checks if entries in dictionary are mat-objects. If yes
    todict is called to change them to nested dictionaries
    '''
    for key in adict:
        if isinstance(adict[key], mat_struct):
            adict[key] = _todict(adict[key])
    return adict        

def _todict(matobj):
    '''
    A recursive function which constructs from matobjects nested dictionaries
    '''
    adict = {}
    for strg in matobj._fieldnames:
        elem = matobj.__dict__[strg]
        if isinstance(elem, mat_struct):
            adict[strg] = _todict(elem)
        else:
            adict[strg] = elem
    return adict



# -*- coding: utf-8 -*-
"""
This file contains TIR one component models
"""
import numpy as np                  # NumPy
import scipy.special as sps

def wixi(x):
    """ Complex Error Function (Faddeeva/Voigt).
        w(i*x) = exp(x**2) * ( 1-erf(x) )
        This function is called by other functions within this module.
        We are using the scipy.special.wofz module which calculates
        w(z) = exp(-z**2) * ( 1-erf(-iz) )
        z = i*x
    """
    z = x*1j
    wixi = sps.wofz(z)
    # We should have a real solution. Make sure nobody complains about
    # some zero-value imaginary numbers.
    
    return np.real_if_close(wixi)

def CF_Gxy_TIR_square(parms, tau):
    # Model 6000
    u""" Two-dimensional diffusion with a square shaped lateral detection
        area taking into account the size of the point spread function.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D      Diffusion coefficient
        [1] σ      Lateral size of the point spread function
                   σ = σ₀ * λ / NA
        [2] a      Side size of the square-shaped detection area
        [3] C_2D   Particle concentration in detection area
        *tau* - lag time

        Please refer to the documentation of PyCorrFit
        for further information on this model function.
        
        Returns: Normalized Lateral correlation function w/square pinhole.
    """
    D = parms[0]
    sigma = parms[1]
    a = parms[2]
    Conc = parms[3]

    var1 = sigma**2+D*tau
    AA = 2*np.sqrt(var1)/(a**2*np.sqrt(np.pi))
    BB = np.exp(-a**2/(4*(var1))) - 1
    CC = sps.erf(a/(2*np.sqrt(var1)))/a
    # gx = AA*BB+CC
    # gxy = gx**2
    # g2D = gxy * eta**2 * Conc
    # F = 1/(eta*Conc)
    # G = g2D / F**2
    G = 1/Conc * (AA*BB+CC)**2
    return G


# 3D free tir
def CF_Gxyz_TIR_square(parms, tau, wixi=wixi):
    # Model 6010
    u""" Three-dimensional diffusion with a square-shaped lateral
        detection area taking into account the size of the
        point spread function; and an exponential decaying profile
        in axial direction.
        
        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D      Diffusion coefficient
        [1] σ      Lateral size of the point spread function
                   σ = σ₀ * λ / NA
        [2] a      Side size of the square-shaped detection area
        [3] d_eva  Evanescent penetration depth
        [4] C_3D   Particle concentration in detection volume
        *tau* - lag time

        Please refer to the documentation of PyCorrFit
        for further information on this model function.
        
        Returns: 3D correlation function for TIR-FCS w/square pinhole
    """
    D = parms[0]
    sigma = parms[1]
    a = parms[2]
    kappa = 1/parms[3]
    Conc = parms[4]
    ### Calculate gxy

    # Axial correlation    
    x = np.sqrt(D*tau)*kappa
    w_ix = wixi(x)
    gz = np.sqrt(D*tau/np.pi) - (2*D*tau*kappa**2 - 1)/(2*kappa) * w_ix

    # Lateral correlation
    gx1 = 2/(a**2*np.sqrt(np.pi)) * np.sqrt(sigma**2+D*tau) * \
          ( np.exp(-a**2/(4*(sigma**2+D*tau))) -1 )
    gx2 = 1/a * sps.erf( a / (2*np.sqrt(sigma**2 + D*tau))) 
    gx =  gx1 + gx2
    gxy = gx**2

    # Non normalized correlation function
    # We do not need eta after normalization
    # g = eta**2 * Conc * gxy * gz
    g = Conc * gxy * gz
    # Normalization:
    # F = eta * Conc / kappa
    F = Conc / kappa
    G = g / F**2
    return G


def MoreInfo_6000(parms, countrate=None):
    u"""Supplementary parameters:
        For a>>sigma, the correlation function at tau=0 corresponds to:
        [4] G(τ=0) = 1/(N_eff) * ( 1-2*σ/(sqrt(π)*a) )²
        Effective detection area:
        [5] A_eff [µm²] = a²
        Effective particle number in detection area:
        [6] N_eff = A_eff * C_2D
    """
    #D = parms[0]
    #sigma = parms[1]
    a = parms[2]
    Conc = parms[3]
    Info=list()

    # Detection area:
    Aeff = a**2 
    # Particel number
    Neff = Aeff * Conc
    # Correlation function at tau = 0
    G_0 = CF_Gxy_TIR_square(parms, 0)

    Info.append(["G(0)", G_0])

    # 10 000 nm² = 0.01 µm²
    # Aeff * 10 000 nm² * 10**(-6)µm²/nm² = Aeff * 0.01 * µm²
    # Have to divide Aeff by 100
    Info.append([u"A_eff [µm²]", Aeff / 100])
    Info.append(["N_eff", Neff])
    if countrate is not None:
        # CPP
        cpp = countrate/Neff
        Info.append(["cpp [kHz]", cpp])
    return Info


def MoreInfo_6010(parms, countrate):
    u"""Supplementary parameters:
        Molarity:
        [5] C_3D [nM] = C_3D [1000/µm³] * 10000/6.0221415
        Effective detection volume:
        [6] V_eff = a² * d_eva
        Effective particle number:
        [7] N_eff = V_eff * C_3D
        For a>>σ, the correlation function at τ=0 corresponds to:
        [8] G(τ=0) = 1/(2*N_eff) * ( 1-2*σ/(sqrt(π)*a) )²
    """
    # 3D Model TIR square
    # 3D TIR (□xσ/exp),Simple 3D diffusion w/ TIR, fct.CF_Gxyz_square_tir
    # D [10 µm²/s],σ [100 nm],a [100 nm],d_eva [100 nm],[conc.] [1000 /µm³]
    #sigma = parms[1]
    a = parms[2]
    d_eva = parms[3]
    conc = parms[4]
    # Sigma

    Info = list()
    # Molarity [nM]:
    # 1000/(µm³)*10**15µm³/l * mol /(6.022*10^23) * 10^9 n
    cmol = conc * 10000/6.0221415
    # Effective volume [al]:
    Veff = a**2 * d_eva
    # Effective particel number
    Neff = a**2 * d_eva * conc
    # Correlation function at tau = 0
    G_0 = CF_Gxyz_TIR_square(parms, 0)

    Info.append(["G(0)", G_0])

    Info.append(["C_3D [nM]", cmol])
    # atto liters
    # 1 000 000 nm³ = 1 al
    Info.append(["V_eff [al]", Veff])
    Info.append(["N_eff", Neff])
    if countrate is not None:
        # CPP
        cpp = countrate/Neff
        Info.append(["cpp [kHz]", cpp])
    return Info



# 2D Model Square
m_twodsq6000 = [6000, u"2D", u"2D diffusion w/ square pinhole",
                CF_Gxy_TIR_square]
labels_6000 = [u"D [10 µm²/s]",
               u"σ [100 nm]",
               u"a [100 nm]",
               u"C_2D [100 /µm²]"]
values_6000 = [0.054, 2.3, 7.5, .6] # [D,lamb,NA,a,conc]
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6000 = [u"D [µm²/s]",
                              u"σ [nm]",
                              u"a [nm]",
                              u"C_2D [1/µm²]"]
values_factor_human_readable_6000 = [10, 100, 100, 100]
valuestofit_6000 = [True, False, False, True]      # Use as fit parameter?
parms_6000 = [labels_6000, values_6000, valuestofit_6000,
              labels_human_readable_6000, values_factor_human_readable_6000]

# 3D Model TIR square
m_3dtirsq6010 = [6010, u"3D", "Simple 3D diffusion w/ TIR",
                 CF_Gxyz_TIR_square]
labels_6010 = [u"D [10 µm²/s]",
               u"σ [100 nm]",
               u"a [100 nm]",
               u"d_eva [100 nm]",
               u"C_3D [1000 /µm³]"]
values_6010 = [0.520, 2.3, 7.5, 1.0, .0216]
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6010 = [u"D [µm²/s]",
                              u"σ [nm]",
                              u"a [nm]",
                              u"d_eva [nm]",
                              u"C_3D [1/µm³]"]
values_factor_human_readable_6010 = [10, 100, 100, 100, 1000]
valuestofit_6010 = [True, False, False, False, True]
parms_6010 = [labels_6010, values_6010, valuestofit_6010,
              labels_human_readable_6010, values_factor_human_readable_6010]



# Pack the models
model1 = dict()
model1["Parameters"] = parms_6000
model1["Definitions"] = m_twodsq6000
model1["Supplements"] = MoreInfo_6000
model1["Verification"] = lambda parms: np.abs(parms)

model2 = dict()
model2["Parameters"] = parms_6010
model2["Definitions"] = m_3dtirsq6010
model2["Supplements"] = MoreInfo_6010
model2["Verification"] = lambda parms: np.abs(parms)


Modelarray = [model1, model2]



# -*- coding: utf-8 -*-
"""
PyCorrFit

Module: user model:
When the user wants to use his own functions.
We are using sympy as function parser instead of writing our own,
which might be safer.
We only parse the function with sympy and test it once during
import. After that, the function is evaluated using eval()!
"""

import codecs
import numpy as np
import scipy.special as sps
import sys
import warnings
try:
    import sympy
    import sympy.functions
    from sympy.core import S
    from sympy.core.function import Function
    from sympy import sympify
except ImportError:
    warnings.warn("Importing sympy failed."+\
                  " Reason: {}.".format(sys.exc_info()[1].message))
    # Define Function, so PyCorrFit will start, even if sympy is not there.
    # wixi needs Function.
    Function = object
import wx

from . import models as mdls


class CorrFunc(object):
    """
        Check the input code of a proposed user model function and
        return a function for fitting via GetFunction.
    """
    def __init__(self, labels, values, substitutes, funcstring):
        self.values = values
        # a --> a
        # b [ms] --> b
        self.variables = list()
        for item in labels:
            self.variables.append(item.split(" ")[0].strip())
        self.funcstring = funcstring
        for key in substitutes.keys():
            # Don't forget to insert the "(" and ")"'s
            self.funcstring = self.funcstring.replace(key, 
                                                       "("+substitutes[key]+")")
            for otherkey in substitutes.keys():
                substitutes[otherkey] = substitutes[otherkey].replace(key, 
                                                       "("+substitutes[key]+")")
        # Convert the function string to a simpification object
        self.simpification = sympify(self.funcstring, sympyfuncdict)
        self.simstring = str(self.simpification)
        self.vardict = evalfuncdict


    def GetFunction(self):
        # Define the function that will be calculated later
        def G(parms, tau):
            tau = np.atleast_1d(tau)
            for i in np.arange(len(parms)):
                self.vardict[self.variables[i]] = float(parms[i])
            self.vardict["tau"] = tau
            # Function called with array/list
            # The problem here might be 
            #for key in vardict.keys():
            #    symstring = symstring.replace(key, str(vardict[key]))
            #symstring = symstring.replace("####", "tau")
            g = eval(self.funcstring, self.vardict)
            ## This would be a safer way to do this, but it is too slow!
            # Once simpy supports arrays, we can use these.
            #
            # g = np.zeros(len(tau))
            # for i in np.arange(len(tau)):
            # vardict["tau"] = tau[i]
            # g[i] = simpification.evalf(subs=vardict)
            return g
        return G


    def TestFunction(self):
        """ Test the function for parsibility with the given parameters.
        """
        vardict = dict()
        for i in np.arange(len(self.variables)):
            vardict[self.variables[i]] = sympify(float(self.values[i]))
        for tau in np.linspace(0.0001, 10000, 10):
            vardict["tau"] = tau
            Number = self.simpification.evalf(subs=vardict)
            if Number.is_Number is False:
                raise SyntaxError("Function could not be parsed!")


class UserModel(object):
    """ Class for importing txt files as models into PyCorrFit.
    """
    def __init__(self, parent):
        " Define all important constants and variables. "
        # Current ID is the last model ID we gave away.
        # This will be set using self.SetCurrentID
        self.CurrentID = None
        # The file to be opened. This is a full path like
        # os.path.join(dirname, filename)
        self.filename = None
        # Imported models
        # Modelarray = [model1, model2]
        self.modelarray = []
        # String that contains the executable code
        self.modelcode = None
        # Parent is main PyCorrFit program
        self.parent = parent
        # The string that identifies the user model menu
        self.UserStr="User"


    def GetCode(self, filename=None):
        """ Get the executable code from the file.
            Optional argument filename may be used. If not self.filename will
            be used.
            This automatically sets self.filename
        """
        if filename is not None:
            self.filename = filename
        openedfile = open(self.filename, 'r')
        code = openedfile.readlines()
        # File should start with a comment #.
        # Remove everything before that comment (BOM).
        startfile = code[0].find("#")
        if startfile != -1:
            code[0] = code[0][startfile:]
        else:
            code[0] = "# "+code[0]
        # Returncode: True if model was imported, False if there was a problem.
        # See ModelImported in class CorrFunc
        self.AddModel(code)
        openedfile.close()


    def AddModel(self, code):
        """ *code* is a list with strings
             each string is one line.
        """
        # a = 1
        # b [ms] = 2.5
        # gAlt = 1+tau/b
        # gProd = a*b
        # G = 1/gA * gB
        labels = list()
        values = list()
        substitutes = dict()
        for line in code:
            # We deal with comments and empty lines
            # We need to check line length first and then we look for
            # a hash.
            line = line.strip()
            if len(line) != 0 and line[0] != "#":
                var, val = line.split("=")
                var = var.strip()
                if var == "G":
                    # Create a fuction that calculates G
                    funcstring = val.strip()
                    self.FuncClass = CorrFunc(labels, values, substitutes,
                                              funcstring)
                    func = self.FuncClass.GetFunction()
                    doc = code[0].strip()
                    # Add whitespaces in model string (looks nicer)
                    for olin in code[1:]:
                        doc = doc + "\n       "+olin.strip()
                    func.func_doc = codecs.decode(doc, "utf-8")
                elif var[0] == "g":
                    substitutes[var] = val.strip()
                else:
                    # Add value and variable to our lists
                    labels.append(codecs.decode(var, "utf-8"))
                    values.append(float(val))
        # Active Parameters we are using for the fitting
        # [0] labels
        # [1] values
        # [2] bool values to fit
        bools = list([False]*len(values))
        bools[0] = True
        # Create Modelarray
        active_parms = [ labels, values, bools ]
        self.SetCurrentID()
        Modelname = code[0][1:].strip()
        definitions = [self.CurrentID, Modelname, Modelname, func]
        model = dict()
        model["Parameters"] = active_parms
        model["Definitions"] = definitions
        self.modelarray.append(model)


    def ImportModel(self):
        """ Do everything that is necessarry to import the models into
            PyCorrFit.
        """
        # Set the model ids of the new model(s)
        # Normally, there is only one model.
        for i in np.arange(len(self.modelarray)):
            self.SetCurrentID()
            self.modelarray[i]["Definitions"][0] = self.CurrentID
        # We assume that the models have the correct ID for now
        mdls.AppendNewModel(self.modelarray)
        # Set variables and models
        # Is this still necessary? - We are doing this for compatibility!
        self.parent.value_set = mdls.values
        self.parent.valuedict = mdls.valuedict
        self.parent.models = mdls.models
        self.parent.modeldict = mdls.modeldict
        self.parent.modeltypes = mdls.modeltypes
        # Get menu
        menu = self.parent.modelmenudict[self.UserStr]
        # Add menu entrys
        for item in self.modelarray:
            # Get definitions
            Defs = item["Definitions"]
            # This is important if we want to save the session with
            # the imported model.
            mdls.modeltypes[self.UserStr].append(Defs[0])
            menuentry = menu.Append(Defs[0], Defs[1], Defs[2])
            self.parent.Bind(wx.EVT_MENU, self.parent.add_fitting_tab,
                             menuentry)


    def TestFunction(self):
        """ Convenience function to test self.FuncClass """
        self.FuncClass.TestFunction()


    def SetCurrentID(self):
        # Check last item or so of modelarray
        # Imported functions get IDs starting from 7000
        theID = 7000
        for model in mdls.models:
            theID = max(theID, model[0])
        self.CurrentID = theID + 1


class wixi(Function):
    """
        This is a ghetto solution for using wofz in sympy.
        It only returns the real part of the function.
        I am not sure, if the eval's are placed correctly.
        I only made it work for my needs. This might be wrong!
        For true use of wofz, I am not using sympy, anyhow.
    """
    nargs = 1
    is_real = True
    @classmethod
    def eval(self, arg):
        return None

    def as_base_exp(self):
        return self, S.One  # @UndefinedVariable

    def _eval_evalf(self, prec):
        result = sps.wofz(1j*float(self.args[0]))
        return sympy.numbers.Number(sympy.functions.re(result))


def evalwixi(x):
    """ Complex Error Function (Faddeeva/Voigt).
        w(i*x) = exp(x**2) * ( 1-erf(x) )
        This function is called by other functions within this module.
        We are using the scipy.special.wofz module which calculates
        w(z) = exp(-z**2) * ( 1-erf(-iz) )
        z = i*x
    """
    z = x*1j
    result = sps.wofz(z)
    # We should have a real solution. Make sure nobody complains about
    # some zero-value imaginary numbers.
    return np.real_if_close(result)



sympyfuncdict = dict()
sympyfuncdict["wixi"] = wixi

evalfuncdict = dict()
evalfuncdict["wixi"] = evalwixi
evalfuncdict["I"] = 1j

scipyfuncs = ['wofz', 'erf', 'erfc']
numpyfuncs = ['abs', 'arccos', 'arcsin', 'arctan', 'arctan2', 'ceil', 'cos',
              'cosh', 'degrees', 'e', 'exp', 'fabs', 'floor', 'fmod', 'frexp',
              'hypot', 'ldexp', 'log', 'log10', 'modf', 'pi', 'power',
              'radians', 'sin', 'sinh', 'sqrt', 'tan', 'tanh']

for func in scipyfuncs:
    evalfuncdict[func] = eval("sps."+func)

for func in numpyfuncs:
    evalfuncdict[func] = eval("np."+func)





# -*- coding: utf-8 -*-
"""
PyCorrFit - module "tools"

This file contains tools, such as dialog boxes and other stuff,
that we need in PyCorrFit.

The tools work with triggers on page updates. Every tool has a 
function `OnPageChanged(self, page, trigger=None)` which is called
when something in the frontend chages. In order to minimize user
stall time, these functions are not executed for a certain list
of triggers that is defined in that function. This e.g. dramatically
speeds up tools like "Statistics view" when batch fitting.
    
Recognized triggers:
 tab_init           : initial stuff that is done for a new page
 tab_browse         : the tab has change and a new page is visible
 fit_batch          : the page is batch-fitted right now
 fit_finalize       : a (batch) fitting process is finished
 parm_batch         : parameters are changed in a batch process
 parm_finalize      : finished (batch) changing of page parameters
 page_add_batch     : when many pages are added at the same time
 page_add_finalize  : finished (batch) adding of pages
"""
# This file is necessary for this folder to become a module that can be 
# imported by PyCorrFit or other people.

import numpy as np                  # NumPy
import sys

from . import datarange
from . import background
from . import overlaycurves
from . import batchcontrol
from . import globalfit
from . import average
from . import simulation

from . import info
from . import statistics
from . import trace
# Load all of the classes
# This also defines the order of the tools in the menu
ImpA = [ 
        ["datarange", "SelectChannels"],
        ["overlaycurves", "Wrapper_Tools"],
        ["batchcontrol", "BatchCtrl"],
        ["globalfit", "GlobalFit"],        
        ["average", "Average"],
        ["background", "BackgroundCorrection"]
       ]

ImpB = [
        ["trace", "ShowTrace"],
        ["statistics", "Stat"],
        ["info", "ShowInfo"],
        ["simulation", "Slide"]
       ]

ModuleActive = list()
ToolsActive = list()
for i in np.arange(len(ImpA)):
    # We have to add "tools." because this is a relative import
    ModuleActive.append(__import__(ImpA[i][0], globals(), locals(), [ImpA[i][1]], -1))
    ToolsActive.append(getattr(ModuleActive[i], ImpA[i][1]))

ModulePassive = list()
ToolsPassive = list()
for i in np.arange(len(ImpB)):
    ModulePassive.append(__import__(ImpB[i][0], globals(), locals(), [ImpB[i][1]], -1))
    ToolsPassive.append(getattr(ModulePassive[i], ImpB[i][1]))
    #ModulePassive.append(importlib.import_module("tools."+ImpB[i][0]))
    #ToolsPassive.append(getattr(ModulePassive[i], ImpB[i][1]))

# This is in the file menu and not needed in the dictionaries below.
from .chooseimport import ChooseImportTypes
from .chooseimport import ChooseImportTypesModel
from .comment import EditComment
# the "special" tool RangeSelector
from .parmrange import RangeSelector

ToolDict = dict()
ToolDict["A"] = ToolsActive
ToolDict["P"] = ToolsPassive


# Make the same for Menu Names in Tools
NameActive = list()
for i in np.arange(len(ImpA)):
    NameActive.append(ModuleActive[i].MENUINFO)
    
NamePassive = list()
for i in np.arange(len(ImpB)):
    NamePassive.append(ModulePassive[i].MENUINFO)


ToolName = dict()
ToolName["A"] = NameActive
ToolName["P"] = NamePassive




# -*- coding: utf-8 -*-
"""
PyCorrFit

Module tools - comment
Edit the sessions' comment.
"""
from __future__ import print_function
import wx


class EditComment(wx.Frame):
    """ Little Dialog to edit the comment on the session. """
    def __init__(self, parent):
        ## Variables
        # parent is main frame
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=parent, title="Session comment",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        initial_size = (400,300)
        initial_sizec = (initial_size[0], initial_size[1]-50)
        self.SetSize(initial_size)
        self.SetMinSize((400,300))
        ## Content
        self.panel = wx.Panel(self)
        self.control = wx.TextCtrl(self.panel, style=wx.TE_MULTILINE, 
                        size=initial_sizec, value=self.parent.SessionComment)
        self.Bind(wx.EVT_TEXT, self.OnTextChanged, self.control)
        text = wx.StaticText(self.panel, 
                   label="Session comments will be saved in the  session file.")
        # buttons
        btnsave = wx.Button(self.panel, wx.ID_SAVE, 'Save Comment')
        self.Bind(wx.EVT_BUTTON, self.OnSave, btnsave)
        #sizers
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        self.topSizer.Add(text)
        self.topSizer.Add(self.control)
        self.topSizer.Add(btnsave, 1, wx.RIGHT | wx.EXPAND)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        #Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)
        wx.EVT_SIZE(self, self.OnSize)
        self.text_changed = False


    def OnSize(self, event):
        size = event.GetSize()
        sizec = (size[0], size[1]-50)
        self.panel.SetSize(size)
        self.control.SetSize(sizec)


    def OnClose(self, e=None):
        self.parent.filemenu.Check(self.parent.menuComm.GetId(), False)
        if self.text_changed:
            # ask the user to save or discard.
            dlg = wx.MessageDialog(self, "Save comment?",
                                   "Do you want to save the current changes?",
                                   style=wx.YES_NO)
            if dlg.ShowModal() == wx.ID_YES:
                self.OnSave()
        self.Destroy()

    def OnTextChanged(self, e=None):
        """ When the user changes the text
        """
        self.text_changed = True

    def OnSave(self, e=None):
        self.parent.SessionComment = self.control.GetValue()
        self.text_changed = False
        self.OnClose()




import numpy as np
import os, sys
#from correlation_methods import *
#from import_methods import *
import time
#from fitting_methods import equation_
#from lmfit import minimize, Parameters,report_fit,report_errors, fit_report

from .correlation_methods import *
from .import_methods import *


"""FCS Bulk Correlation Software

    Copyright (C) 2015  Dominic Waithe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

class picoObject():
    #This is the class which holds the .pt3 data and parameters
    def __init__(self,filepath, par_obj,fit_obj):
    
        #parameter object and fit object. If 
        self.par_obj = par_obj
        self.fit_obj = fit_obj
        self.type = 'mainObject'
        
        #self.PIE = 0
        self.filepath = str(filepath)
        self.nameAndExt = os.path.basename(self.filepath).split('.')
        self.name = self.nameAndExt[0]
        self.par_obj.data.append(filepath);
        self.par_obj.objectRef.append(self)
        
        #Imports pt3 file format to object.
        self.unqID = self.par_obj.numOfLoaded
        
        #For fitting.
        self.objId1 = None
        self.objId2 = None
        self.objId3 = None
        self.objId4 = None
        self.processData();
        
        self.plotOn = True;


    def processData(self):

        self.NcascStart = self.par_obj.NcascStart
        self.NcascEnd = self.par_obj.NcascEnd
        self.Nsub = self.par_obj.Nsub
        self.winInt = self.par_obj.winInt
        self.photonCountBin = self.par_obj.photonCountBin
        
        #File import 
        self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = pt3import(self.filepath)
        
        #Colour assigned to file.
        self.color = self.par_obj.colors[self.unqID % len(self.par_obj.colors)]

        #How many channels there are in the files.
        self.numOfCH =  np.unique(np.array(self.subChanArr)).__len__()-1 #Minus 1 because not interested in channel 15.
        #TODO Generates the interleaved excitation channel if required. 
        #if (self.aug == 'PIE'):
            #self.pulsedInterleavedExcitation()
        
        #Finds the numbers which address the channels.
        self.ch_present = np.unique(np.array(self.subChanArr[0:100]))

        #Calculates decay function for both channels.
        self.photonDecayCh1,self.decayScale1 = delayTime2bin(np.array(self.dTimeArr),np.array(self.subChanArr),self.ch_present[0],self.winInt)
        
        if self.numOfCH ==  2:
            self.photonDecayCh2,self.decayScale2 = delayTime2bin(np.array(self.dTimeArr),np.array(self.subChanArr),self.ch_present[1],self.winInt)

        #Time series of photon counts. For visualisation.
        self.timeSeries1,self.timeSeriesScale1 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[0],self.photonCountBin)
        if self.numOfCH ==  2:
            self.timeSeries2,self.timeSeriesScale2 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[1],self.photonCountBin)

        
        #Calculates the Auto and Cross-correlation functions.
        self.crossAndAuto(np.array(self.trueTimeArr),np.array(self.subChanArr))
        
        
           
       
        
        if self.fit_obj != None:
            #If fit object provided then creates fit objects.
            if self.objId1 == None:
                corrObj= corrObject(self.filepath,self.fit_obj);
                self.objId1 = corrObj.objId
                self.fit_obj.objIdArr.append(corrObj.objId)
                self.objId1.name = self.name+'_CH0_Auto_Corr'
                self.objId1.ch_type = 0 #channel 0 Auto
                self.objId1.prepare_for_fit()
            self.objId1.autoNorm = np.array(self.autoNorm[:,0,0]).reshape(-1)
            self.objId1.autotime = np.array(self.autotime).reshape(-1)
            self.objId1.param = self.fit_obj.def_param
            
            
            if self.numOfCH ==  2:
                if self.objId3 == None:
                    corrObj= corrObject(self.filepath,self.fit_obj);
                    self.objId3 = corrObj.objId
                    self.fit_obj.objIdArr.append(corrObj.objId)
                    self.objId3.name = self.name+'_CH1_Auto_Corr'
                    self.objId3.ch_type = 1 #channel 1 Auto
                    self.objId3.prepare_for_fit()
                self.objId3.autoNorm = np.array(self.autoNorm[:,1,1]).reshape(-1)
                self.objId3.autotime = np.array(self.autotime).reshape(-1)
                self.objId3.param = self.fit_obj.def_param
                
                if self.objId2 == None:
                    corrObj= corrObject(self.filepath,self.fit_obj);
                    self.objId2 = corrObj.objId
                    self.fit_obj.objIdArr.append(corrObj.objId)
                    self.objId2.name = self.name+'_CH01_Cross_Corr'
                    self.objId2.ch_type = 2 #01cross
                    self.objId2.prepare_for_fit()
                self.objId2.autoNorm = np.array(self.autoNorm[:,0,1]).reshape(-1)
                self.objId2.autotime = np.array(self.autotime).reshape(-1)
                self.objId2.param = self.fit_obj.def_param
                

                if self.objId4 == None:
                    corrObj= corrObject(self.filepath,self.fit_obj);
                    self.objId4 = corrObj.objId
                    self.fit_obj.objIdArr.append(corrObj.objId)
                    self.objId4.name = self.name+'_CH10_Cross_Corr'
                    self.objId4.ch_type = 3 #10cross
                    self.objId4.prepare_for_fit()
                self.objId4.autoNorm = np.array(self.autoNorm[:,1,0]).reshape(-1)
                self.objId4.autotime = np.array(self.autotime).reshape(-1)
                self.objId4.param = self.fit_obj.def_param
                
            self.fit_obj.fill_series_list()
        self.dTimeMin = 0
        self.dTimeMax = np.max(self.dTimeArr)
        self.subDTimeMin = self.dTimeMin
        self.subDTimeMax = self.dTimeMax
        del self.subChanArr 
        del self.trueTimeArr 
        del self.dTimeArr
    def crossAndAuto(self,trueTimeArr,subChanArr):
        #For each channel we loop through and find only those in the correct time gate.
        #We only want photons in channel 1 or two.
        y = trueTimeArr[subChanArr < 3]
        validPhotons = subChanArr[subChanArr < 3 ]


        #Creates boolean for photon events in either channel.
        num = np.zeros((validPhotons.shape[0],2))
        num[:,0] = (np.array([np.array(validPhotons) ==self.ch_present[0]])).astype(np.int32)
        if self.numOfCH ==2:
            num[:,1] = (np.array([np.array(validPhotons) ==self.ch_present[1]])).astype(np.int32)


        self.count0 = np.sum(num[:,0]) 
        self.count1 = np.sum(num[:,1])

        t1 = time.time()
        auto, self.autotime = tttr2xfcs(y,num,self.NcascStart,self.NcascEnd, self.Nsub)
        t2 = time.time()
        print 'timing',t2-t1
        

        #Normalisation of the TCSPC data:
        maxY = np.ceil(max(self.trueTimeArr))
        self.autoNorm = np.zeros((auto.shape))
        self.autoNorm[:,0,0] = ((auto[:,0,0]*maxY)/(self.count0*self.count0))-1
        
        if self.numOfCH ==  2:
            self.autoNorm[:,1,1] = ((auto[:,1,1]*maxY)/(self.count1*self.count1))-1
            self.autoNorm[:,1,0] = ((auto[:,1,0]*maxY)/(self.count1*self.count0))-1
            self.autoNorm[:,0,1] = ((auto[:,0,1]*maxY)/(self.count0*self.count1))-1
            

        #Normalisaation of the decay functions.
        self.photonDecayCh1Min = self.photonDecayCh1-np.min(self.photonDecayCh1)
        self.photonDecayCh1Norm = self.photonDecayCh1Min/np.max(self.photonDecayCh1Min)
        
        
        if self.numOfCH ==  2:
            self.photonDecayCh2Min = self.photonDecayCh2-np.min(self.photonDecayCh2)
            self.photonDecayCh2Norm = self.photonDecayCh2Min/np.max(self.photonDecayCh2Min)
        
        return 
   

    
    
class subPicoObject():
    def __init__(self,parentId,xmin,xmax,TGid,par_obj):
        #Binning window for decay function
        self.TGid = TGid
        #Parameters for auto-correlation and cross-correlation.
        self.parentId = parentId
        self.par_obj = par_obj
        self.NcascStart = self.parentId.NcascStart
        self.NcascEnd = self.parentId.NcascEnd
        self.Nsub = self.parentId.Nsub
        self.fit_obj = self.parentId.fit_obj
        
        self.type = 'subObject'
        #Appends the object to the subObject register.
        self.par_obj.subObjectRef.append(self)
        self.unqID = self.par_obj.subNum
        self.parentUnqID = self.parentId.unqID
        #self.chanArr = parentObj.chanArr
        #self.trueTimeArr = self.parentId.trueTimeArr
        #self.dTimeArr = self.parentId.dTimeArr
        self.color = self.parentId.color
        self.numOfCH = self.parentId.numOfCH
        self.ch_present = self.parentId.ch_present

        self.filepath = str(self.parentId.filepath)
        self.xmin = xmin
        self.xmax = xmax

        self.nameAndExt = os.path.basename(self.filepath).split('.')
        self.name = 'TG-'+str(self.unqID)+'-xmin_'+str(round(xmin,0))+'-xmax_'+str(round(xmax,0))+'-'+self.nameAndExt[0]

        self.objId1 = None
        self.objId2 = None
        self.objId3 = None
        self.objId4 = None
        self.processData();
        self.plotOn = True
        
        
    def processData(self):
        self.NcascStart= self.par_obj.NcascStart
        self.NcascEnd= self.par_obj.NcascEnd
        self.Nsub = self.par_obj.Nsub
        self.winInt = self.par_obj.winInt
        
        
        self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = pt3import(self.filepath)
        
        
        
        self.subArrayGeneration(self.xmin,self.xmax,np.array(self.subChanArr))
        
        


        self.dTimeMin = self.parentId.dTimeMin
        self.dTimeMax = self.parentId.dTimeMax
        self.subDTimeMin = self.dTimeMin
        self.subDTimeMax = self.dTimeMax
        

        
        #Adds names to the fit function for later fitting.
        if self.objId1 == None:
            corrObj= corrObject(self.filepath,self.fit_obj);
            self.objId1 = corrObj.objId
            self.fit_obj.objIdArr.append(corrObj.objId)
            self.objId1.name = self.name+'_CH0_Auto_Corr'
            self.objId1.ch_type = 0 #channel 0 Auto
            self.objId1.prepare_for_fit()
        self.objId1.autoNorm = np.array(self.autoNorm[:,0,0]).reshape(-1)
        self.objId1.autotime = np.array(self.autotime).reshape(-1)
        self.objId1.param = self.fit_obj.def_param
        
        
        if self.numOfCH ==2:
            if self.objId3 == None:
                corrObj= corrObject(self.filepath,self.fit_obj);
                self.objId3 = corrObj.objId
                self.fit_obj.objIdArr.append(corrObj.objId)
                self.objId3.name = self.name+'_CH1_Auto_Corr'
                self.objId3.ch_type = 1 #channel 1 Auto
                self.objId3.prepare_for_fit()
            self.objId3.autoNorm = np.array(self.autoNorm[:,1,1]).reshape(-1)
            self.objId3.autotime = np.array(self.autotime).reshape(-1)
            self.objId3.param = self.fit_obj.def_param
            if self.objId2 == None:
                corrObj= corrObject(self.filepath,self.fit_obj);
                self.objId2 = corrObj.objId
                self.fit_obj.objIdArr.append(corrObj.objId)
                self.objId2.name = self.name+'_CH01_Cross_Corr'
                self.objId2.ch_type = 2 #channel 01 Cross
                self.objId2.prepare_for_fit()
            self.objId2.autoNorm = np.array(self.autoNorm[:,0,1]).reshape(-1)
            self.objId2.autotime = np.array(self.autotime).reshape(-1)
            self.objId2.param = self.fit_obj.def_param
            if self.objId4 == None:
                corrObj= corrObject(self.filepath,self.fit_obj);
                self.objId4 = corrObj.objId
                self.fit_obj.objIdArr.append(corrObj.objId)
                self.objId4.name = self.name+'_CH10_Cross_Corr'
                self.objId4.ch_type = 3 #channel 10 Cross
                self.objId4.prepare_for_fit()
            self.objId4.autoNorm = np.array(self.autoNorm[:,1,0]).reshape(-1)
            self.objId4.autotime = np.array(self.autotime).reshape(-1)
            self.objId4.param = self.fit_obj.def_param
            
        
        self.fit_obj.fill_series_list()  
        del self.subChanArr 
        del self.trueTimeArr 
        del self.dTimeArr 
    


    def subArrayGeneration(self,xmin,xmax,subChanArr):
        if(xmax<xmin):
            xmin1 = xmin
            xmin = xmax
            xmax = xmin1
        #self.subChanArr = np.array(self.chanArr)
        #Finds those photons which arrive above certain time or below certain time.
        photonInd = np.logical_and(self.dTimeArr>=xmin, self.dTimeArr<=xmax).astype(np.bool)
        
        subChanArr[np.invert(photonInd).astype(np.bool)] = 16
        
        self.crossAndAuto(subChanArr)

        return
    def crossAndAuto(self,subChanArr):
        #We only want photons in channel 1 or two.
        validPhotons = subChanArr[subChanArr < 3]
        y = self.trueTimeArr[subChanArr < 3]
        #Creates boolean for photon events in either channel.
        num = np.zeros((validPhotons.shape[0],2))
        num[:,0] = (np.array([np.array(validPhotons) ==self.ch_present[0]])).astype(np.int)
        if self.numOfCH == 2:
            num[:,1] = (np.array([np.array(validPhotons) ==self.ch_present[1]])).astype(np.int)

        self.count0 = np.sum(num[:,0]) 
        self.count1 = np.sum(num[:,1]) 
        #Function which calculates auto-correlation and cross-correlation.



        auto, self.autotime = tttr2xfcs(y,num,self.NcascStart,self.NcascEnd, self.Nsub)

        maxY = np.ceil(max(self.trueTimeArr))
        self.autoNorm = np.zeros((auto.shape))
        self.autoNorm[:,0,0] = ((auto[:,0,0]*maxY)/(self.count0*self.count0))-1
        if self.numOfCH ==2:
            self.autoNorm[:,1,1] = ((auto[:,1,1]*maxY)/(self.count1*self.count1))-1
            self.autoNorm[:,1,0] = ((auto[:,1,0]*maxY)/(self.count1*self.count0))-1
            self.autoNorm[:,0,1] = ((auto[:,0,1]*maxY)/(self.count0*self.count1))-1

        return 

class corrObject():
    def __init__(self,filepath,parentFn):
        #the container for the object.
        self.parentFn = parentFn
        self.type = 'corrObject'
        self.filepath = str(filepath)
        self.nameAndExt = os.path.basename(self.filepath).split('.')
        self.name = self.nameAndExt[0]
        self.ext = self.nameAndExt[-1]
        self.autoNorm=[]
        self.autotime=[]
        self.model_autoNorm =[]
        self.model_autotime = []
        self.datalen= []
        self.objId = self;
        self.param = []
        self.goodFit = True
        self.fitted = False
        self.checked = False
        self.toFit = False
       
        #main.data.append(filepath);
        #The master data object reference 
        #main.corrObjectRef.append(self)
        #The id in terms of how many things are loaded.
        #self.unqID = main.label.numOfLoaded;
        #main.label.numOfLoaded = main.label.numOfLoaded+1
    def prepare_for_fit(self):
        if self.parentFn.ch_check_ch0.isChecked() == True and self.ch_type == 0:
            self.toFit = True
        if self.parentFn.ch_check_ch1.isChecked() == True and self.ch_type == 1:
            self.toFit = True
            
        if self.parentFn.ch_check_ch01.isChecked() == True and self.ch_type == 2:
            self.toFit = True
        if self.parentFn.ch_check_ch10.isChecked() == True and self.ch_type == 3:
            self.toFit = True
        #self.parentFn.modelFitSel.clear()
        #for objId in self.parentFn.objIdArr:
         #   if objId.toFit == True:
          #      self.parentFn.modelFitSel.addItem(objId.name)
        self.parentFn.updateFitList()
    def residual(self, param, x, data,options):
    
        A = equation_(param, x,options)
        residuals = data-A
        return residuals
    def fitToParameters(self):
        self.parentFn.updateParamFirst()
        self.parentFn.updateTableFirst()
        self.parentFn.updateParamFirst()
        

        #convert line coordinate
        
        #Find the index of the nearest point in the scale.
        
        data = np.array(self.autoNorm).astype(np.float64).reshape(-1)
        scale = np.array(self.autotime).astype(np.float64).reshape(-1)
        indx_L = int(np.argmin(np.abs(scale -  self.parentFn.dr.xpos)))
        indx_R = int(np.argmin(np.abs(scale -  self.parentFn.dr1.xpos)))
        

        res = minimize(self.residual, self.param, args=(scale[indx_L:indx_R+1],data[indx_L:indx_R+1], self.parentFn.def_options))
        self.residualVar = res.residual
        output = fit_report(self.param)
        print 'residual',res.chisqr
        if(res.chisqr>0.05):
            print 'CAUTION DATA DID NOT FIT WELL CHI^2 >0.05',res.chisqr
            self.goodFit = False
        else:
            self.goodFit = True
        self.fitted = True
        self.chisqr = res.chisqr
        rowArray =[];
        localTime = time.asctime( time.localtime(time.time()) )
        rowArray.append(str(self.name))  
        rowArray.append(str(localTime))
        rowArray.append(str(self.parentFn.diffModEqSel.currentText()))
        rowArray.append(str(self.parentFn.def_options['Diff_species']))
        rowArray.append(str(self.parentFn.tripModEqSel.currentText()))
        rowArray.append(str(self.parentFn.def_options['Triplet_species']))
        rowArray.append(str(self.parentFn.dimenModSel.currentText()))
        rowArray.append(str(scale[indx_L]))
        rowArray.append(str(scale[indx_R]))

        for key, value in self.param.iteritems() :
            rowArray.append(str(value.value))
            rowArray.append(str(value.stderr))
            if key =='GN0':
                try:
                    rowArray.append(str(1/value.value))
                except:
                    rowArray.append(str(0))
        
        self.rowText = rowArray
        
        self.parentFn.updateTableFirst();
        self.model_autoNorm = equation_(self.param, scale[indx_L:indx_R+1],self.parentFn.def_options)
        self.model_autotime = scale[indx_L:indx_R+1]
        self.parentFn.on_show()

        #self.parentFn.axes.plot(model_autotime,model_autoNorm, 'o-')
        #self.parentFn.canvas.draw();
    
    def load_from_file(self,channel):
        tscale = [];
        tdata = [];
        if self.ext == 'SIN':
            self.parentFn.objIdArr.append(self.objId)
            proceed = False
            
            for line in csv.reader(open(self.filepath, 'rb'),delimiter='\t'):
                
                if proceed ==True:
                    if line ==[]:
                        break;
                    
                    
                    tscale.append(float(line[0]))
                    tdata.append(float(line[channel+1]))
                else:
                  
                  if (str(line)  == "[\'[CorrelationFunction]\']"):
                    proceed = True;
            

            self.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
            self.autotime= np.array(tscale).astype(np.float64).reshape(-1)*1000
            self.name = self.name+'-CH'+str(channel)
            self.ch_type = channel;
            self.prepare_for_fit()


            self.param = self.parentFn.def_param
            self.parentFn.fill_series_list()
            
        
                    #Where we add the names.


        if self.ext == 'csv':
            
            self.parentFn.objIdArr.append(self)
            
            c = 0
            
            for line in csv.reader(open(self.filepath, 'rb')):
                if (c >0):
                    tscale.append(line[0])
                    tdata.append(line[1])
                c +=1;

            self.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
            self.autotime= np.array(tscale).astype(np.float64).reshape(-1)
            self.ch_type = 0
            self.datalen= len(tdata)
            self.objId.prepare_for_fit()








# -*- coding: utf-8 -*-
"""
This file contains some simple 2D models for confocal microscopy
"""
from __future__ import division

import numpy as np


# 2D simple gauss
def CF_Gxy_gauss(parms, tau):
    u""" Two-dimensional diffusion with a Gaussian laser profile.

        G(τ) = offset + 1/( n * (1+τ/τ_diff) )
    
        Calculation of diffusion coefficient and concentration
        from the effective radius of the detection profile (r₀ = 2*σ):
        D = r₀²/(4*τ_diff)
        Conc = n/(π*r₀²)

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal area
        [1] τ_diff  Characteristic residence time in confocal area
        [3] offset
        *tau* - lag time
    """
    n = parms[0]
    taudiff = parms[1]
    dc = parms[2]

    BB = 1 / ( (1.+tau/taudiff) )
    G = dc + 1/n * BB
    return G

def Check_xy_gauss(parms):
    parms[0] = np.abs(parms[0])
    parms[1] = np.abs(parms[1])

    return parms


# 2D simple gauss
def CF_Gxy_T_gauss(parms, tau):
    u""" Two-dimensional diffusion with a Gaussian laser profile,
        including a triplet component.
        The triplet factor takes into account a blinking term.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        triplet = 1 + T/(1-T)*exp(-τ/τ_trip)

        G(τ) = offset + 1/( n * (1+τ/τ_diff) )*triplet
    
        Calculation of diffusion coefficient and concentration
        from the effective radius of the detection profile (r₀ = 2*σ):
        D = r₀²/(4*τ_diff)
        Conc = n/(π*r₀²)

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal area
        [1] τ_diff  Characteristic residence time in confocal area
        [2] τ_trip  Characteristic residence time in triplet state
        [3] T       Fraction of particles in triplet (non-fluorescent) state
                    0 <= T < 1
        [4] offset
        *tau* - lag time
    """
    n = parms[0]
    taudiff = parms[1]
    tautrip = parms[2]
    T = parms[3]
    dc = parms[4]

    if tautrip == 0 or T == 0:
        triplet = 1
    else:
        triplet = 1 + T/(1-T) * np.exp(-tau/tautrip)

    BB = 1 / ( (1.+tau/taudiff) )
    G = dc + 1/n * BB * triplet
    return G



def Check_xy_T_gauss(parms):
    parms[0] = np.abs(parms[0])
    parms[1] = np.abs(parms[1]) # = taudiff
    tautrip = np.abs(parms[2])
    T=parms[3]


    # Triplet fraction is between 0 and one. T may not be one!
    T = (0.<=T<1.)*T + .99999999999999*(T>=1)

    parms[2] = tautrip
    parms[3] = T
    return parms



# 2D + 2D + Triplet Gauß
    # Model 6031
def CF_Gxyz_gauss_2D2DT(parms, tau):
    u""" Two-component, two-dimensional diffusion with a Gaussian laser
        profile, including a triplet component.
        The triplet factor takes into account blinking according to triplet
        states of excited molecules.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        particle1 = F₁/(1+τ/τ₁)
        particle2 = α*(1-F₁)/(1+τ/τ₂)
        triplet = 1 + T/(1-T)*exp(-τ/τ_trip)
        norm = (F₁ + α*(1-F₁))²
        G = 1/n*(particle1 + particle2)*triplet/norm + offset

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal area
                    (n = n₁+n₂)
        [1] τ₁      Diffusion time of particle species 1
        [2] τ₂      Diffusion time of particle species 2
        [3] F₁      Fraction of molecules of species 1 (n₁ = n*F₁)
                    0 <= F₁ <= 1
        [4] α       Relative molecular brightness of particle 2
                    compared to particle 1 (α = q₂/q₁)
        [5] τ_trip  Characteristic residence time in triplet state
        [6] T       Fraction of particles in triplet (non-fluorescent)
                    state 0 <= T < 1
        [7] offset
        *tau* - lag time
    """
    n=parms[0]
    taud1=parms[1]
    taud2=parms[2]
    F=parms[3]
    alpha=parms[4]
    tautrip=parms[5]
    T=parms[6]
    off=parms[7]

    particle1 = F/( 1+tau/taud1 )
    particle2 = alpha**2*(1-F)/( 1+tau/taud2 )
    # If the fraction of dark molecules is zero, we put the
    # whole triplet fraction to death.
    triplet = 1 + T/(1-T)*np.exp(-tau/tautrip)
    # For alpha == 1, *norm* becomes one
    norm = (F + alpha*(1-F))**2

    G = 1/n*(particle1 + particle2)*triplet/norm + off
    return G

def Check_6031(parms):
    parms[0] = np.abs(parms[0])
    parms[1] = np.abs(parms[1]) # = taud1
    parms[2] = np.abs(parms[2]) # = taud2
    F=parms[3]
    parms[4] = np.abs(parms[4])
    tautrip = np.abs(parms[5])
    T=parms[6]
    #off=parms[7]
    
     
    # Triplet fraction is between 0 and one. T may not be one!
    T = (0.<=T<1.)*T + .99999999999999*(T>=1)
    # Fraction of molecules may also be one
    F = (0.<=F<=1.)*F + 1.*(F>1)

    parms[3] = F
    parms[5] = tautrip
    parms[6] = T

    return parms


def MoreInfo_6001(parms, countrate=None):
    # We can only give you the effective particle number
    n = parms[0]
    Info = list()
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append(["cpp [kHz]", cpp])
    return Info
    
    
def MoreInfo_6031(parms, countrate=None):
    u"""Supplementary parameters:
        [8] n₁ = n*F₁     Particle number of species 1
        [9] n₂ = n*(1-F₁) Particle number of species 2
    """
    # We can only give you the effective particle number
    n = parms[0]
    F1 = parms[3]
    Info = list()
    # The enumeration of these parameters is very important for
    # plotting the normalized curve. Countrate must come out last!
    Info.append([u"n\u2081", n*F1])
    Info.append([u"n\u2082", n*(1.-F1)])
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append(["cpp [kHz]", cpp])
    return Info


# 2D Model Gauss
m_twodga6001 = [6001, u"2D", u"2D confocal diffusion", CF_Gxy_gauss]
labels_6001 = [u"n",
               u"τ_diff [ms]",
               u"offset"]
values_6001 = [4.0, 0.4, 0.0]
valuestofit_6001 = [True, True, False]
parms_6001 = [labels_6001, values_6001, valuestofit_6001]


# 2D Model Gauss with Triplet
m_twodga6002 = [6002, u"T+2D", u"2D confocal diffusion with triplet", 
                CF_Gxy_T_gauss]
labels_6002 = [u"n",
               u"τ_diff [ms]",
               u"τ_trip [ms]",
               u"T",
               u"offset"]
values_6002 = [4.0, 0.4, 0.001, 0.01, 0.0]
labels_hr_6002 = [u"n",
                  u"τ_diff [ms]",
                  u"τ_trip [µs]",
                  u"T",
                  u"offset"]
factors_hr_6002 = [1., 1., 1000., 1., 1.]
valuestofit_6002 = [True, True, True, True, False]
parms_6002 = [labels_6002, values_6002, valuestofit_6002,
              labels_hr_6002, factors_hr_6002]


# 2D + 2D + T model gauss
m_gauss_2d_2d_t_mix_6031 = [6031, u"T+2D+2D",
                            u"Separate 2D diffusion + triplet, Gauß",
                            CF_Gxyz_gauss_2D2DT]
labels_6031  = ["n",
                u"τ"+u"\u2081"+u" [ms]",
                u"τ"+u"\u2082"+u" [ms]",
                u"F"+u"\u2081", 
                u"\u03b1"+u" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                u"τ_trip [ms]",
                u"T",
                u"offset"
                ]
values_6031 = [ 
                25,      # n
                5,       # taud1
                1000,    # taud2
                0.5,     # F
                1.0,     # alpha
                0.001,   # tautrip
                0.01,    # T
                0.0      # offset
                ]        
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6031  = [
                        u"n",
                        u"τ"+u"\u2081"+u" [ms]",
                        u"τ"+u"\u2082"+u" [ms]",
                        u"F"+u"\u2081", 
                        u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                        u"τ_trip [µs]",
                        u"T",
                        u"offset"
                            ]
values_factor_human_readable_6031 = [
                1.,     # "n",
                1.,     # "τ"+u"\u2081"+" [ms]",
                1.,     # "τ"+u"\u2082"+" [ms]",
                1.,     # "F"+u"\u2081", 
                1.,     # u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                1000.,  # "τ_trip [µs]",
                1.,     # "T",
                1.      # "offset"
                ]
valuestofit_6031 = [True, True, True, True, False, False, False, False]
parms_6031 = [labels_6031, values_6031, valuestofit_6031,
              labels_human_readable_6031, values_factor_human_readable_6031]


model1 = dict()
model1["Parameters"] = parms_6001
model1["Definitions"] = m_twodga6001
model1["Supplements"] = MoreInfo_6001
model1["Verification"] = Check_xy_gauss

model2 = dict()
model2["Parameters"] = parms_6002
model2["Definitions"] = m_twodga6002
model2["Supplements"] = MoreInfo_6001
model2["Verification"] = Check_xy_T_gauss

model3 = dict()
model3["Parameters"] = parms_6031
model3["Definitions"] = m_gauss_2d_2d_t_mix_6031
model3["Supplements"] = MoreInfo_6031
model3["Verification"] = Check_6031


Modelarray = [model1, model2, model3]



import struct
import numpy as np


"""FCS Bulk Correlation Software

    Copyright (C) 2015  Dominic Waithe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

def pt3import(filepath):
    """The file import for the .pt3 file"""
    f = open(filepath, 'rb')
    Ident = f.read(16)
    FormatVersion = f.read(6)
    CreatorName = f.read(18)
    CreatorVersion = f.read(12)
    FileTime = f.read(18)
    CRLF = f.read(2)
    CommentField = f.read(256)
    Curves = struct.unpack('i', f.read(4))[0]
    BitsPerRecord = struct.unpack('i', f.read(4))[0]
    RoutingChannels = struct.unpack('i', f.read(4))[0]
    NumberOfBoards = struct.unpack('i', f.read(4))[0]
    ActiveCurve = struct.unpack('i', f.read(4))[0]
    MeasurementMode = struct.unpack('i', f.read(4))[0]
    SubMode = struct.unpack('i', f.read(4))[0]
    RangeNo = struct.unpack('i', f.read(4))[0]
    Offset = struct.unpack('i', f.read(4))[0]
    AcquisitionTime = struct.unpack('i', f.read(4))[0]
    StopAt = struct.unpack('i', f.read(4))[0]
    StopOnOvfl = struct.unpack('i', f.read(4))[0]
    Restart = struct.unpack('i', f.read(4))[0]
    DispLinLog = struct.unpack('i', f.read(4))[0]
    DispTimeFrom = struct.unpack('i', f.read(4))[0]
    DispTimeTo = struct.unpack('i', f.read(4))[0]
    DispCountFrom = struct.unpack('i', f.read(4))[0]
    DispCountTo = struct.unpack('i', f.read(4))[0]
    DispCurveMapTo = [];
    DispCurveShow =[];
    for i in range(0,8):
        DispCurveMapTo.append(struct.unpack('i', f.read(4))[0]);
        DispCurveShow.append(struct.unpack('i', f.read(4))[0]);
    ParamStart =[];
    ParamStep =[];
    ParamEnd =[];
    for i in range(0,3):
        ParamStart.append(struct.unpack('i', f.read(4))[0]);
        ParamStep.append(struct.unpack('i', f.read(4))[0]);
        ParamEnd.append(struct.unpack('i', f.read(4))[0]);
        
    RepeatMode = struct.unpack('i', f.read(4))[0]
    RepeatsPerCurve = struct.unpack('i', f.read(4))[0]
    RepeatTime = struct.unpack('i', f.read(4))[0]
    RepeatWait = struct.unpack('i', f.read(4))[0]
    ScriptName = f.read(20)

    #The next is a board specific header

    HardwareIdent = f.read(16)
    HardwareVersion = f.read(8)
    HardwareSerial = struct.unpack('i', f.read(4))[0]
    SyncDivider = struct.unpack('i', f.read(4))[0]

    CFDZeroCross0 = struct.unpack('i', f.read(4))[0]
    CFDLevel0 = struct.unpack('i', f.read(4))[0]
    CFDZeroCross1 = struct.unpack('i', f.read(4))[0]
    CFDLevel1 = struct.unpack('i', f.read(4))[0]

    Resolution = struct.unpack('f', f.read(4))[0]

    #below is new in format version 2.0

    RouterModelCode      = struct.unpack('i', f.read(4))[0]
    RouterEnabled        = struct.unpack('i', f.read(4))[0]

    #Router Ch1
    RtChan1_InputType    = struct.unpack('i', f.read(4))[0]
    RtChan1_InputLevel   = struct.unpack('i', f.read(4))[0]
    RtChan1_InputEdge    = struct.unpack('i', f.read(4))[0]
    RtChan1_CFDPresent   = struct.unpack('i', f.read(4))[0]
    RtChan1_CFDLevel     = struct.unpack('i', f.read(4))[0]
    RtChan1_CFDZeroCross = struct.unpack('i', f.read(4))[0]
    #Router Ch2
    RtChan2_InputType    = struct.unpack('i', f.read(4))[0]
    RtChan2_InputLevel   = struct.unpack('i', f.read(4))[0]
    RtChan2_InputEdge    = struct.unpack('i', f.read(4))[0]
    RtChan2_CFDPresent   = struct.unpack('i', f.read(4))[0]
    RtChan2_CFDLevel     = struct.unpack('i', f.read(4))[0]
    RtChan2_CFDZeroCross = struct.unpack('i', f.read(4))[0]
    #Router Ch3
    RtChan3_InputType    = struct.unpack('i', f.read(4))[0]
    RtChan3_InputLevel   = struct.unpack('i', f.read(4))[0]
    RtChan3_InputEdge    = struct.unpack('i', f.read(4))[0]
    RtChan3_CFDPresent   = struct.unpack('i', f.read(4))[0]
    RtChan3_CFDLevel     = struct.unpack('i', f.read(4))[0]
    RtChan3_CFDZeroCross = struct.unpack('i', f.read(4))[0]
    #Router Ch4
    RtChan4_InputType    = struct.unpack('i', f.read(4))[0]
    RtChan4_InputLevel   = struct.unpack('i', f.read(4))[0]
    RtChan4_InputEdge    = struct.unpack('i', f.read(4))[0]
    RtChan4_CFDPresent   = struct.unpack('i', f.read(4))[0]
    RtChan4_CFDLevel     = struct.unpack('i', f.read(4))[0]
    RtChan4_CFDZeroCross = struct.unpack('i', f.read(4))[0]

    #The next is a T3 mode specific header.
    ExtDevices = struct.unpack('i', f.read(4))[0]

    Reserved1 = struct.unpack('i', f.read(4))[0]
    Reserved2 = struct.unpack('i', f.read(4))[0]
    CntRate0 = struct.unpack('i', f.read(4))[0]
    CntRate1 = struct.unpack('i', f.read(4))[0]

    StopAfter = struct.unpack('i', f.read(4))[0]
    StopReason = struct.unpack('i', f.read(4))[0]
    Records = struct.unpack('i', f.read(4))[0]
    ImgHdrSize =struct.unpack('i', f.read(4))[0]

    #Special Header for imaging.
    if ImgHdrSize > 0:
        ImgHdr = struct.unpack('i', f.read(ImgHdrSize))[0]
    ofltime = 0;

    cnt_1=0; cnt_2=0; cnt_3=0; cnt_4=0; cnt_Ofl=0; cnt_M=0; cnt_Err=0; # just counters
    WRAPAROUND=65536;

    #Put file Save info here.

    syncperiod = 1e9/CntRate0;
    #outfile stuff here.
    #fpout.
    #T3RecordArr = [];
    
    chanArr = [0]*Records
    trueTimeArr =[0]*Records
    dTimeArr=[0]*Records
    #f1=open('./testfile', 'w+')
    for b in range(0,Records):
        T3Record = struct.unpack('I', f.read(4))[0];
        
        #T3RecordArr.append(T3Record)
        nsync = T3Record & 65535
        chan = ((T3Record >> 28) & 15);
        chanArr[b]=chan
        #f1.write(str(i)+" "+str(T3Record)+" "+str(nsync)+" "+str(chan)+" ")
        dtime = 0;
        
        if chan == 1:
            cnt_1 = cnt_1+1;dtime = ((T3Record >> 16) & 4095);#f1.write(str(dtime)+" ")
        elif chan == 2: 
            cnt_2 = cnt_2+1;dtime = ((T3Record >> 16) & 4095);#f1.write(str(dtime)+" ")
        elif chan == 3: 
            cnt_3 = cnt_3+1;dtime = ((T3Record >> 16) & 4095);#f1.write(str(dtime)+" ")
        elif chan == 4: 
            cnt_4 = cnt_4+1;dtime = ((T3Record >> 16) & 4095);#f1.write(str(dtime)+" ")
        elif chan == 15:
            markers = ((T3Record >> 16) & 15);
            
            if markers ==0:
                ofltime = ofltime +WRAPAROUND;
                cnt_Ofl = cnt_Ofl+1
                #f1.write("Ofl "+" ")
            else:
                cnt_M=cnt_M+1
                #f1.write("MA:%1u "+markers+" ")
            
        truensync = ofltime + nsync;
        truetime = (truensync * syncperiod) + (dtime*Resolution);
        trueTimeArr[b] = truetime
        dTimeArr[b] = dtime
        #f1.write(str(truensync)+" "+str(truetime)+"\n")
    f.close();
    #f1.close();
    
    
    
    return np.array(chanArr), np.array(trueTimeArr), np.array(dTimeArr), Resolution


# -*- coding: utf-8 -*-
"""
PyCorrFit

Documentation and program specific information
"""


import sys

# This is a fake class for modules not available.
class Fake(object):
    def __init__(self):
        self.__version__ = "N/A"
        self.version = "N/A"
        self.use = lambda x: None        
try:
    import matplotlib
except:
    # Create fake opbject for matplotlib
    matplotlib = Fake()
    
# We do catch warnings about performing this before matplotlib.backends stuff
#matplotlib.use('WXAgg') # Tells matplotlib to use WxWidgets
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    matplotlib.use('WXAgg') # Tells matplotlib to use WxWidgets for dialogs
import numpy
import os
import platform
import scipy

try:
    import sympy
except ImportError:
    print " Warning: module sympy not found!"
    sympy = Fake()

import wx
import yaml

import readfiles


def GetLocationOfFile(filename):
    dirname = os.path.dirname(os.path.abspath(__file__))
    locations = [
                    os.path.realpath(dirname+"/../"),
                    os.path.realpath(dirname+"/../pycorrfit_doc/"),
                    os.path.realpath(dirname+"/../doc/"),
                ]

    for i in range(len(locations)):
        # check /usr/lib64/32 -> /usr/lib
        for larch in ["lib32", "lib64"]:
            if dirname.count(larch):
                locations.append(locations[i].replace(larch, "lib", 1))
    
    ## freezed binaries:
    if hasattr(sys, 'frozen'):
        try:
            adir = sys._MEIPASS + "/doc/"  # @UndefinedVariable
        except:
            adir = "./"
        locations.append(os.path.realpath(adir))
    for loc in locations:
        thechl = os.path.join(loc,filename)
        if os.path.exists(thechl):
            return thechl
            break
    # if this does not work:
    return None


def GetLocationOfChangeLog(filename = "ChangeLog.txt"):
    return GetLocationOfFile(filename)


def GetLocationOfDocumentation(filename = "PyCorrFit_doc.pdf"):
    """ Returns the location of the documentation if there is any."""
    return GetLocationOfFile(filename)


def info(version):
    """ Returns a little info about our program and what it can do.
    """
    textwin = u"""
    Copyright 2011-2012 Paul Mueller, Biotec - TU Dresden

    A versatile tool for fitting and analyzing correlation curves.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 1000 /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 um^2/s
    unit of inverse area: 100 /um^2
    unit of inv. volume : 1000 /um^3 """
    textlin = u"""
    © 2011-2012 Paul Müller, Biotec - TU Dresden

    A versatile tool for fitting and analyzing correlation curves.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 1000 /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³ """
    if platform.system() != 'Linux':
        texta = textwin
    else:
        texta = textlin
    one = u"    PyCorrFit version "+version+"\n\n"
    two = u"\n\n    Supported file types:"
    keys = readfiles.Filetypes.keys()
    keys.sort()
    for item in keys:
        if item.split("|")[0] != readfiles.Allsupfilesstring:
            two = two + "\n     - "+item.split("|")[0]
    lizenz = ""
    for line in licence().splitlines():
        lizenz += "    "+line+"\n"
    return one + lizenz + texta + two

    
def licence():
    return """PyCorrFit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published 
by the Free Software Foundation, either version 2 of the License, 
or (at your option) any later version.

PyCorrFit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def SoftwareUsed():
    """ Return some Information about the software used for this program """
    text = "Python "+sys.version+\
           "\n\nModules:"+\
           "\n - cython "+\
           "\n - matplotlib "+matplotlib.__version__+\
           "\n - NumPy "+numpy.__version__+\
           "\n - PyYAML "+yaml.__version__ +\
           "\n - SciPy "+scipy.__version__+\
           "\n - sympy "+sympy.__version__ +\
           "\n - wxPython "+wx.__version__
    # Other software
    text += "\n\nOther software:"+\
            "\n - FCS_point_correlator (9311a5c15e)" +\
            "\n    PicoQuant file format for Python by Dominic Waithe"
    if hasattr(sys, 'frozen'):
        pyinst = "\n\nThis executable has been created using PyInstaller."
        text += pyinst
        if 'Anaconda' in sys.version or "Continuum Analytics" in sys.version:
            conda = "\n\nPowered by Anaconda"
            text += conda
    return text


# Standard homepage
HomePage = "http://pycorrfit.craban.de/"
# Changelog filename
ChangeLog = "ChangeLog.txt"
StaticChangeLog = GetLocationOfChangeLog(ChangeLog)

# Check if we can extract the version
try:
    clfile = open(StaticChangeLog, 'r')
    __version__ = clfile.readline().strip()
    clfile.close()     
except:
    __version__ = "0.0.0-unknown"
    
    
# Github homepage
GitChLog = "https://raw.github.com/paulmueller/PyCorrFit/master/ChangeLog.txt"
GitHome = "https://github.com/paulmueller/PyCorrFit"
GitWiki = "https://github.com/paulmueller/PyCorrFit/wiki"



# -*- coding: utf-8 -*-
""" PyCorrFit loader
"""
from os.path import dirname, abspath, split

import sys
sys.path = [split(abspath(dirname(__file__)))[0]] + sys.path

import pycorrfit
pycorrfit.Main()



# -*- coding: utf-8 -*-
"""
PyCorrFit - Module openfile

This file contains definitions for opening PyCorrFit sessions and
saving PyCorrFit correlation curves.
"""

import codecs
import csv
import numpy as np
import os
import shutil
import tempfile
import yaml
import zipfile
import warnings

from . import doc

# These imports are required for loading data
from .readfiles import Filetypes  # @UnusedImport
from .readfiles import BGFiletypes  # @UnusedImport
from .fcs_data_set import Trace


def LoadSessionData(sessionfile, parameters_only=False):
    """ Load PyCorrFit session data from a zip file (.pcfs)


    Parameters
    ----------
    sessionfile : str
        File from which data will be loaded
    parameters_only : bool
        Only load the parameters from the YAML file
    
    
    Returns
    -------
    Infodict : dict
        Infodict may contain the following keys:
        "Backgrounds", list: contains the backgrounds
        "Comments", dict: "Session" comment and int keys to Page titles
        "Correlations", dict: page numbers, all correlation curves
        "External Functions", dict: modelids to external model functions
        "External Weights", dict: page numbers, external weights for fitting
        "Parameters", dict: page numbers, all parameters of the pages
        "Preferences", dict: not used yet
        "Traces", dict: page numbers, all traces of the pages
        "Version", str: the PyCorrFit version of the session
    """
    Infodict = dict()
    # Get the version
    Arc = zipfile.ZipFile(sessionfile, mode='r')
    readmefile = Arc.open("Readme.txt")
    # e.g. "This file was created using PyCorrFit version 0.7.6"
    Infodict["Version"] = readmefile.readline()[46:].strip()
    readmefile.close()
    # Get the yaml parms dump:
    yamlfile = Arc.open("Parameters.yaml")
    # Parameters: Fitting and drawing parameters of correlation curve
    # The *yamlfile* is responsible for the order of the Pages #i.
    Infodict["Parameters"] = yaml.safe_load(yamlfile)
    yamlfile.close()
    if parameters_only:
        Arc.close()
        return Infodict
    # Supplementary data (errors of fit)
    supname = "Supplements.yaml"
    try:
        Arc.getinfo(supname)
    except:
        pass
    else:
        supfile = Arc.open(supname)
        supdata = yaml.safe_load(supfile)
        Infodict["Supplements"] = dict()
        for idp in supdata:
            Infodict["Supplements"][idp[0]] = dict() 
            Infodict["Supplements"][idp[0]]["FitErr"] = idp[1]
            if len(idp) > 2:
                # As of version 0.7.4 we save chi2 and shared pages -global fit
                Infodict["Supplements"][idp[0]]["Chi sq"] = idp[2]
                Infodict["Supplements"][idp[0]]["Global Share"] = idp[3]
    ## Preferences: Reserved for a future version of PyCorrFit :)
    prefname = "Preferences.yaml"
    try:
        Arc.getinfo(prefname)
    except KeyError:
        pass
    else:
        yamlpref = Arc.open(prefname)
        Infodict["Preferences"] = yaml.safe_load(yamlpref)
        yamlpref.close()
    # Get external functions
    Infodict["External Functions"] = dict()
    key = 7001
    while key <= 7999:
        # (There should not be more than 1000 functions)
        funcfilename = "model_"+str(key)+".txt"
        try:
            Arc.getinfo(funcfilename)
        except KeyError:
            # No more functions to import
            key = 8000
        else:
            funcfile =  Arc.open(funcfilename)
            Infodict["External Functions"][key] = funcfile.read()
            funcfile.close()
            key=key+1
    # Get the correlation arrays
    Infodict["Correlations"] = dict()
    for i in np.arange(len(Infodict["Parameters"])):
        # The *number* is used to identify the correct file
        number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
        pageid = int(number)
        expfilename = "data"+number+".csv"
        expfile = Arc.open(expfilename, 'r')
        readdata = csv.reader(expfile, delimiter=',')
        dataexp = list()
        tau = list()
        if str(readdata.next()[0]) == "# tau only":
            for row in readdata:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    tau.append(float(row[0]))
            tau = np.array(tau)
            dataexp = None
        else:
            for row in readdata:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    dataexp.append((float(row[0]), float(row[1])))
            dataexp = np.array(dataexp)
            tau = dataexp[:,0]
        Infodict["Correlations"][pageid] = [tau, dataexp]
        del readdata
        expfile.close()
    # Get the Traces
    Infodict["Traces"] = dict()
    for i in np.arange(len(Infodict["Parameters"])):
        # The *number* is used to identify the correct file
        number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
        pageid = int(number)
        # Find out, if we have a cross correlation data type
        IsCross = False
        try:
            IsCross = Infodict["Parameters"][i][7]
        except IndexError:
            # No Cross correlation
            pass
        if IsCross is False:
            tracefilenames = ["trace"+number+".csv"]
        else:
            # Cross correlation uses two traces
            tracefilenames = ["trace"+number+"A.csv",
                              "trace"+number+"B.csv" ]
        thistrace = list()
        for tracefilename in tracefilenames:
            try:
                Arc.getinfo(tracefilename)
            except KeyError:
                pass
            else:
                tracefile = Arc.open(tracefilename, 'r')
                traceread = csv.reader(tracefile, delimiter=',')
                singletrace = list()
                for row in traceread:
                    # Exclude commentaries
                    if (str(row[0])[0:1] != '#'):
                        singletrace.append((float(row[0]), float(row[1])))
                singletrace = np.array(singletrace)
                thistrace.append(singletrace)
                del traceread
                del singletrace
                tracefile.close()
        if len(thistrace) != 0:
            Infodict["Traces"][pageid] = thistrace
        else:
            Infodict["Traces"][pageid] = None
    # Get the comments, if they exist
    commentfilename = "comments.txt"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(commentfilename)
    except KeyError:
        pass   
    else:
        # Open the file
        commentfile = Arc.open(commentfilename, 'r')
        Infodict["Comments"] = dict()
        for i in np.arange(len(Infodict["Parameters"])):
            number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
            pageid = int(number)
            # Strip line ending characters for all the Pages.
            Infodict["Comments"][pageid] = commentfile.readline().strip()
        # Now Add the Session Comment (the rest of the file). 
        ComList = commentfile.readlines()
        Infodict["Comments"]["Session"] = ''
        for line in ComList:
            Infodict["Comments"]["Session"] += line
        commentfile.close()
    # Get the Backgroundtraces and data if they exist
    bgfilename = "backgrounds.csv"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(bgfilename)
    except KeyError:
        pass
    else:
        # Open the file
        Infodict["Backgrounds"] = list()
        bgfile = Arc.open(bgfilename, 'r')
        bgread = csv.reader(bgfile, delimiter='\t')
        i = 0
        for bgrow in bgread:
            bgtracefilename = "bg_trace"+str(i)+".csv"
            bgtracefile = Arc.open(bgtracefilename, 'r')
            bgtraceread = csv.reader(bgtracefile, delimiter=',')
            bgtrace = list()
            for row in bgtraceread:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    bgtrace.append((np.float(row[0]), np.float(row[1])))
            bgtrace = np.array(bgtrace)
            newbackground = Trace(trace=bgtrace, name=str(bgrow[1]), countrate=np.float(bgrow[0]))
            Infodict["Backgrounds"].append(newbackground)
            i = i + 1
        bgfile.close()
    # Get external weights if they exist
    WeightsFilename = "externalweights.txt"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(WeightsFilename)
    except:
        pass
    else:
        Wfile = Arc.open(WeightsFilename, 'r')
        Wread = csv.reader(Wfile, delimiter='\t')
        Weightsdict = dict()
        for wrow in Wread:
            Pkey = wrow[0]  # Page of weights
            pageid = int(Pkey)
            # Do not overwrite anything
            try:
                Weightsdict[pageid]
            except:
                Weightsdict[pageid] = dict()
            Nkey = wrow[1]  # Name of weights
            Wdatafilename = "externalweights_data"+Pkey+"_"+Nkey+".csv"
            Wdatafile = Arc.open(Wdatafilename, 'r')
            Wdatareader = csv.reader(Wdatafile)
            Wdata = list()
            for row in Wdatareader:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    Wdata.append(np.float(row[0]))
            Weightsdict[pageid][Nkey] = np.array(Wdata)
        Infodict["External Weights"] = Weightsdict
    Arc.close()
    return Infodict


def SaveSessionData(sessionfile, Infodict):
    """ Session PyCorrFit session data to file.
    
    
    Parameters
    ----------
    sessionfile : str
        The suffix ".pcfs" is automatically appended.
    Infodict : dict
        Infodict may contain the following keys:
        "Backgrounds", list: contains the backgrounds
        "Comments", dict: "Session" comment and int keys to Page titles
        "Correlations", dict: page numbers, all correlation curves
        "External Functions, dict": modelids to external model functions
        "External Weights", dict: page numbers, external weights for fitting
        "Parameters", dict: page numbers, all parameters of the pages
        "Preferences", dict: not used yet
        "Traces", dict: page numbers, all traces of the pages


    The version of PyCorrFit is written to Readme.txt
    """
    (dirname, filename) = os.path.split(sessionfile)
    # Sometimes you have multiple endings...
    if filename.endswith(".pcfs") is not True:
        filename += ".pcfs"
    # Change working directory
    returnWD = os.getcwd()
    tempdir = tempfile.mkdtemp()
    os.chdir(tempdir)
    # Create zip file
    Arc = zipfile.ZipFile(filename, mode='w')
    # Only do the Yaml thing for safe operations.
    # Make the yaml dump
    parmsfilename = "Parameters.yaml"
    # Parameters have to be floats in lists
    # in order for yaml.safe_load to work.
    Parms =  Infodict["Parameters"]
    ParmsKeys = Parms.keys()
    ParmsKeys.sort()
    Parmlist = list()
    for idparm in ParmsKeys:
        # Make sure we do not accidently save arrays.
        # This would not work correctly with yaml.
        Parms[idparm][2] = np.array(Parms[idparm][2],dtype="float").tolist()
        Parms[idparm][3] = np.array(Parms[idparm][3],dtype="bool").tolist()
        # Range of fitting parameters
        Parms[idparm][9] = np.array(Parms[idparm][9],dtype="float").tolist()
        Parmlist.append(Parms[idparm])
    yaml.dump(Parmlist, open(parmsfilename, "wb"))
    Arc.write(parmsfilename)
    os.remove(os.path.join(tempdir, parmsfilename))
    # Supplementary data (errors of fit)
    errsfilename = "Supplements.yaml"
    Sups =  Infodict["Supplements"]
    SupKeys = Sups.keys()
    SupKeys.sort()
    Suplist = list()
    for idsup in SupKeys:
        error = Sups[idsup]["FitErr"]
        chi2 = Sups[idsup]["Chi sq"]
        globalshare = Sups[idsup]["Global Share"]
        Suplist.append([idsup, error, chi2, globalshare])
    yaml.dump(Suplist, open(errsfilename, "wb"))
    Arc.write(errsfilename)
    os.remove(os.path.join(tempdir, errsfilename))
    # Save external functions
    for key in Infodict["External Functions"].keys():
        funcfilename = "model_"+str(key)+".txt"
        funcfile =  codecs.open(funcfilename, 'w', encoding="utf-8")
        funcfile.write(Infodict["External Functions"][key])
        funcfile.close()
        Arc.write(funcfilename)
        os.remove(os.path.join(tempdir, funcfilename))
    # Save (dataexp and tau)s into separate csv files.
    for pageid in Infodict["Correlations"].keys():
        # Since *Array* and *Parms* are in the same order (the page order),
        # we will identify the filename by the Page title number.
        number = str(pageid)
        expfilename = "data"+number+".csv"
        expfile = open(expfilename, 'wb')
        tau = Infodict["Correlations"][pageid][0]
        exp = Infodict["Correlations"][pageid][1]
        dataWriter = csv.writer(expfile, delimiter=',')
        if exp is not None:
            # Names of Columns
            dataWriter.writerow(['# tau', 'experimental data'])
            # Actual Data
            # Do not use len(tau) instead of len(exp[:,0])) !
            # Otherwise, the experimental data will not be saved entirely,
            # if it has been cropped. Because tau might be smaller, than
            # exp[:,0] --> tau = exp[startcrop:endcrop,0]
            for j in np.arange(len(exp[:,0])):
                dataWriter.writerow(["%.20e" % exp[j,0],
                                     "%.20e" % exp[j,1]])
        else:
            # Only write tau
            dataWriter.writerow(['# tau'+' only'])
            for j in np.arange(len(tau)):
                dataWriter.writerow(["%.20e" % tau[j]])
        expfile.close()
        # Add to archive
        Arc.write(expfilename)
        os.remove(os.path.join(tempdir, expfilename))
    # Save traces into separate csv files.
    for pageid in Infodict["Traces"].keys():
        number = str(pageid)
        # Since *Trace* and *Parms* are in the same order, which is the
        # Page order, we will identify the filename by the Page title 
        # number.
        if Infodict["Traces"][pageid] is not None and len(Infodict["Traces"][pageid]) != 0:
            if Parms[pageid][7] is True:
                # We have cross correlation: save two traces
                ## A
                tracefilenamea = "trace"+number+"A.csv"
                tracefile = open(tracefilenamea, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][0][:,0]
                rate = Infodict["Traces"][pageid][0][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilenamea)
                os.remove(os.path.join(tempdir, tracefilenamea))
                ## B
                tracefilenameb = "trace"+number+"B.csv"
                tracefile = open(tracefilenameb, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][1][:,0]
                rate = Infodict["Traces"][pageid][1][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilenameb)
                os.remove(os.path.join(tempdir, tracefilenameb))
            else:
                # Save one single trace
                tracefilename = "trace"+number+".csv"
                tracefile = open(tracefilename, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][0][:,0]
                rate = Infodict["Traces"][pageid][0][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilename)
                os.remove(os.path.join(tempdir, tracefilename))
    # Save comments into txt file
    commentfilename = "comments.txt"
    commentfile = open(commentfilename, 'wb')
    # Comments[-1] is comment on whole Session
    Ckeys = Infodict["Comments"].keys()
    Ckeys.sort()
    for key in Ckeys:
        if key != "Session":
            commentfile.write(Infodict["Comments"][key]+"\r\n")
    commentfile.write(Infodict["Comments"]["Session"])
    commentfile.close()
    Arc.write(commentfilename)
    os.remove(os.path.join(tempdir, commentfilename))
    ## Save Background information:
    Background = Infodict["Backgrounds"]
    if len(Background) > 0:
        # We do not use a comma separated, but a tab separated file,
        # because a comma might be in the name of a bg.
        bgfilename = "backgrounds.csv"
        bgfile = open(bgfilename, 'wb')
        bgwriter = csv.writer(bgfile, delimiter='\t')
        for i in np.arange(len(Background)):
            bgwriter.writerow([str(Background[i].countrate), Background[i].name])
            # Traces
            bgtracefilename = "bg_trace"+str(i)+".csv"
            bgtracefile = open(bgtracefilename, 'wb')
            bgtraceWriter = csv.writer(bgtracefile, delimiter=',')
            bgtraceWriter.writerow(['# time', 'count rate'])
            # Actual Data
            time = Background[i][:,0]
            rate = Background[i][:,1]
            for j in np.arange(len(time)):
                bgtraceWriter.writerow(["%.20e" % time[j],
                                        "%.20e" % rate[j]])
            bgtracefile.close()
            # Add to archive
            Arc.write(bgtracefilename)
            os.remove(os.path.join(tempdir, bgtracefilename))
        bgfile.close()
        Arc.write(bgfilename)
        os.remove(os.path.join(tempdir, bgfilename))
    ## Save External Weights information
    WeightedPageID = Infodict["External Weights"].keys()
    WeightedPageID.sort()
    WeightFilename = "externalweights.txt"
    WeightFile = open(WeightFilename, 'wb')
    WeightWriter = csv.writer(WeightFile, delimiter='\t')
    for pageid in WeightedPageID:
        number = str(pageid)
        NestWeights = Infodict["External Weights"][pageid].keys()
        # The order of the types does not matter, since they are
        # sorted in the frontend and upon import. We sort them here, anyhow.
        NestWeights.sort()
        for Nkey in NestWeights:
            WeightWriter.writerow([number, str(Nkey).strip()])
            # Add data to a File
            WeightDataFilename = "externalweights_data"+number+\
                                 "_"+str(Nkey).strip()+".csv"
            WeightDataFile = open(WeightDataFilename, 'wb')
            WeightDataWriter = csv.writer(WeightDataFile)
            wdata = Infodict["External Weights"][pageid][Nkey]
            for jw in np.arange(len(wdata)):
                WeightDataWriter.writerow([str(wdata[jw])])
            WeightDataFile.close()
            Arc.write(WeightDataFilename)
            os.remove(os.path.join(tempdir, WeightDataFilename))
    WeightFile.close()
    Arc.write(WeightFilename)
    os.remove(os.path.join(tempdir, WeightFilename))
    ## Readme
    rmfilename = "Readme.txt"
    rmfile = open(rmfilename, 'wb')
    rmfile.write(ReadmeSession)
    rmfile.close()
    Arc.write(rmfilename)
    os.remove(os.path.join(tempdir, rmfilename))
    # Close the archive
    Arc.close()
    # Move archive to destination directory
    shutil.move(os.path.join(tempdir, filename), 
                os.path.join(dirname, filename) )
    # Go to destination directory
    os.chdir(returnWD)
    os.rmdir(tempdir)


def ExportCorrelation(exportfile, Page, info, savetrace=True):
    """ Write correlation data to a file
        
    
    Parameters
    ----------
    exportfile : str
        Absolute filename to save data
    Page : PyCorrFit Page object
        Contains all correlation data
    info : module
        The `info` tool module. This is a workaround until Page has
        its own class to create info data.
    savetrace : bool
        Append the trace to the file
    """

    openedfile = codecs.open(exportfile, 'w', encoding="utf-8")
    ## First, some doc text
    openedfile.write(ReadmeCSV.replace('\n', '\r\n'))
    # The infos
    InfoMan = info.InfoClass(CurPage=Page)
    PageInfo = InfoMan.GetCurFancyInfo()
    for line in PageInfo.splitlines():
        openedfile.write(u"# "+line+"\r\n")
    openedfile.write(u"#\r\n#\r\n")
    # Get all the data we need from the Page
    # Modeled data
    corr = Page.corr
    mod = corr.modeled_plot[:,1]
    if corr.correlation is not None:
        # Experimental data
        tau = corr.correlation_fit[:,0]
        exp = corr.correlation_fit[:,1]
        res = corr.residuals_fit[:,1]
        # Plotting! Because we only export plotted area.
        
        if corr.is_weighted_fit:
            weightname = corr.fit_weight_type
            try:
                weight = corr.fit_results["fit weights"]
            except KeyError:
                weight = corr.fit_weight_data
    
            if weight is None:
                pass
            
            elif len(weight) != len(exp):
                text = "Weights have not been calculated for the "+\
                       "area you want to export. Pressing 'Fit' "+\
                       "again should solve this issue. Weights will "+\
                       "not be saved."
                warnings.warn(text)
                weight = None
        else:
            weight = None
            weightname = None
    else:
        tau = corr.lag_time_fit
        exp = None
        res = None
    # Include weights in data saving:
    # PyCorrFit thinks in [ms], but we will save as [s]
    timefactor = 0.001
    tau = timefactor * tau
    ## Now we want to write all that data into the file
    # This is for csv writing:
    ## Correlation curve
    dataWriter = csv.writer(openedfile, delimiter='\t')
    if exp is not None:
        header = '# Lag time [s]'+"\t"+ \
                 'Experimental correlation'+"\t"+ \
                 'Fitted correlation'+ "\t"+ \
                 'Residuals'+"\r\n"
        data = [tau, exp, mod, res]
        if corr.is_weighted_fit and weight is not None:
            header = "{} \t Weights [{}] \r\n".format(
                      header.strip(), weightname)
            data.append(weight)
    else:
        header = '# Lag time [s]'+"\t"+ \
                 'Correlation function'+"\r\n"
        data = [tau, mod]
    # Write header
    openedfile.write(header)
    # Write data
    for i in np.arange(len(data[0])):
        # row-wise, data may have more than two elements per row
        datarow = list()
        for j in np.arange(len(data)):
            rowcoli = "{:.10e}".format(data[j][i])
            datarow.append(rowcoli)
        dataWriter.writerow(datarow)
    ## Trace
    # Only save the trace if user wants us to:
    if savetrace:
        # We will also save the trace in [s]
        # Intensity trace in kHz may stay the same
        if len(corr.traces) > 0:
            # Mark beginning of Trace
            openedfile.write('#\r\n#\r\n# BEGIN TRACE\r\n#\r\n')
            # Columns
            time = corr.traces[0][:,0]*timefactor
            intensity = corr.traces[0][:,1]
            # Write
            openedfile.write('# Time [s]'+"\t" 
                                 'Intensity trace [kHz]'+" \r\n")
            for i in np.arange(len(time)):
                dataWriter.writerow(["{:.10e}".format(time[i]),
                                     "{:.10e}".format(intensity[i])])
        if len(corr.traces) > 1:
            # We have some cross-correlation here:
            # Mark beginning of Trace B
            openedfile.write('#\r\n#\r\n# BEGIN SECOND TRACE\r\n#\r\n')
            # Columns
            time = corr.traces[1][:,0]*timefactor
            intensity = corr.traces[1][:,1]
            # Write
            openedfile.write('# Time [s]'+"\t" 
                                 'Intensity trace [kHz]'+" \r\n")
            for i in np.arange(len(time)):
                dataWriter.writerow(["{:.10e}".format(time[i]),
                                     "{:.10e}".format(intensity[i])])

        openedfile.close()


session_wildcards = [".pcfs", ".pycorrfit-session.zip"]


ReadmeCSV = """# This file was created using PyCorrFit version {}.
#
# Lines starting with a '#' are treated as comments.
# The data is stored as CSV below this comment section.
# Data usually consists of lag times (channels) and
# the corresponding correlation function - experimental
# and fitted values plus resulting residuals.
# If this file is opened by PyCorrFit, only the first two
# columns will be imported as experimental data.
#
""".format(doc.__version__)
    
    
ReadmeSession = """This file was created using PyCorrFit version {}.
The .zip archive you are looking at is a stored session of PyCorrFit.
If you are interested in how the data is stored, you will find
out here. Most important are the dimensions of units:
Dimensionless representation:
 unit of time        : 1 ms
 unit of inverse time: 10³ /s
 unit of distance    : 100 nm
 unit of Diff.coeff  : 10 µm²/s
 unit of inverse area: 100 /µm²
 unit of inv. volume : 1000 /µm³
From there, the dimension of any parameter may be
calculated.

There are a number of files within this archive, 
depending on what was done during the session.

backgrounds.csv
 - Contains the list of backgrounds used and
 - Averaged intensities in [kHz]

bg_trace*.csv (where * is an integer)
 - The trace of the background corresponding
   to the line number in backgrounds.csv
 - Time in [ms], Trace in [kHz]

comments.txt
 - Contains page titles and session comment
 - First n lines are titles, rest is session
   comment (where n is total number of pages)

data*.csv (where * is (Number of page))
 - Contains lag times [ms]
 - Contains experimental data, if available

externalweights.txt
 - Contains names (types) of external weights other than from
   Model function or spline fit
 - Linewise: 1st element is page number, 2nd is name
 - According to this data, the following files are present in the archive

externalweights_data_*PageID*_*Type*.csv
 - Contains weighting information of Page *PageID* of type *Type*

model_*ModelID*.txt
 - An external (user-defined) model file with internal ID *ModelID*

Parameters.yaml
 - Contains all Parameters for each page
   Block format:
    - - '#(Number of page): '       
      - (Internal model ID)
      - (List of parameters)
      - (List of checked parameters (for fitting))
      - [(Min channel selected), (Max channel selected)]
      - [(Weighted fit method (0=None, 1=Spline, 2=Model function)), 
         (No. of bins from left and right),
         (No. of knots (of e.g. spline)),
         (Type of fitting algorithm (e.g. "Lev-Mar", "Nelder-Mead")]
      - [B1,B2] Background to use (line in backgrounds.csv)
           B2 is always *null* for autocorrelation curves
      - Data type is Cross-correlation?
      - Parameter id (int) used for normalization in plotting.
        This number first enumerates the model parameters and then
        the supplemental parameters (e.g. "n1").
      - - [min, max] fitting parameter range of 1st parameter
        - [min, max] fitting parameter range of 2nd parameter
        - etc.
 - Order in Parameters.yaml defines order of pages in a session
 - Order in Parameters.yaml defines order in comments.txt

Readme.txt (this file)

Supplements.yaml
 - Contains errors of fitting
   Format:
   -- Page number
    -- [parameter id, error value]
     - [parameter id, error value]
    - Chi squared
    - [pages that share parameters] (from global fitting)
     
trace*.csv (where * is (Number of page) | appendix "A" or "B" point to
            the respective channels (only in cross-correlation mode))
 - Contains times [ms]
 - Contains countrates [kHz]
""".format(doc.__version__)



# -*- coding: utf-8 -*-
"""
methods to open CSV files
"""
import os
import csv
import numpy as np


def openCSV(dirname, filename):
    """ 
    Read relevant data from a file looking like this:
        [...]
        # Comment
        # Data type: Autocorrelation
        [...]
        1.000000e-006   3.052373e-001
        1.020961e-006   3.052288e-001
        1.042361e-006   3.052201e-001
        1.064209e-006   3.052113e-001
        1.086516e-006   3.052023e-001
        1.109290e-006   3.051931e-001
        [...]
        # BEGIN TRACE
        [...]
        10.852761   31.41818
        12.058624   31.1271
        13.264486   31.27305
        14.470348   31.33442
        15.676211   31.15861
        16.882074   31.08564
        18.087936   31.21335
        [...]

    The correlation part could also look like this:
        # Channel (tau [s])    Experimental correlation    Fitted correlation    Residuals      Weights [model function]
        2.0000000000e-07    1.5649271000e-01    1.5380094370e-01    2.6917663029e-03    7.3158300646e-03
        4.0000000000e-07    1.4751239000e-01    1.5257959602e-01    -5.0672060199e-03    5.8123579098e-03
        6.0000000000e-07    1.5145113000e-01    1.5137624642e-01    7.4883584881e-05    8.5622019656e-03
        8.0000000000e-07    1.5661088000e-01    1.5019053433e-01    6.4203456659e-03    6.8098486549e-03
        1.0000000000e-06    1.5456273000e-01    1.4902210818e-01    5.5406218229e-03    7.2476381023e-03
        1.2000000000e-06    1.3293905000e-01    1.4787062503e-01    -1.4931575028e-02    6.9861494246e-03
        1.4000000000e-06    1.4715790000e-01    1.4673575040e-01    4.2214960494e-04    6.9810206017e-03
        1.6000000000e-06    1.5247520000e-01    1.4561715797e-01    6.8580420325e-03    6.6680066656e-03
        1.8000000000e-06    1.4703974000e-01    1.4451452937e-01    2.5252106284e-03    6.3299717550e-03
    In that case we are also importing the weights.

    Data type:
    If Data type is "Cross-correlation", we will try to import
    two traces after "# BEGIN SECOND TRACE"

    1st section:
     First column denotes tau in seconds and the second row the
     correlation signal.
    2nd section:
     First column denotes tau in seconds and the second row the
     intensity trace in kHz.


    Returns:
    1. A list with tuples containing two elements:
       1st: tau in ms
       2nd: corresponding correlation signal
    2. None - usually is the trace, but the trace is not saved in
              the PyCorrFit .csv format.
    3. A list with one element, indicating, that we are opening only
       one correlation curve.
    """
    # Check if the file is correlation data
    csvfile = open(os.path.join(dirname, filename), 'r')
    firstline = csvfile.readline()
    if firstline.lower().count("this is not correlation data") > 0:
        csvfile.close()
        return None
    csvfile.close()
    
    # Define what will happen to the file
    timefactor = 1000 # because we want ms instead of s
    csvfile = open(os.path.join(dirname, filename), 'r')
    readdata = csv.reader(csvfile, delimiter=',')
    data = list()
    weights = list()
    weightname = "external"
    trace = None
    traceA = None
    DataType="AC" # May be changed
    numtraces = 0
    prev_row = None
    for row in readdata:
        if len(row) == 0 or len(str(row[0]).strip()) == 0:
            # Do nothing with empty/whitespace lines
            pass
            # Beware that the len(row) statement has to be called first
            # (before the len(str(row[0]).strip()) ). Otherwise some
            # error would be raised.
        elif str(row[0])[:12].lower() == "# Type AC/CC".lower():
            corrtype = str(row[0])[12:].strip().strip(":").strip()
            if corrtype[:17].lower() == "cross-correlation":
                # We will later try to import a second trace
                DataType="CC"
                DataType += corrtype[17:].strip()
            elif corrtype[0:15].lower() == "autocorrelation":
                DataType="AC"
                DataType += corrtype[15:].strip()         
        elif str(row[0])[0:13].upper() == '# BEGIN TRACE':
            # Correlation is over. We have a trace
            corr = np.array(data)
            data=list()
            numtraces = 1
        elif str(row[0])[0:20].upper() == '# BEGIN SECOND TRACE':
            # First trace is over. We have a second trace
            traceA = np.array(data)
            data = list()
            numtraces = 2
        # Exclude commentaries
        elif str(row[0])[0:1] != '#':
            # Read the 1st section
            # On Windows we had problems importing nan values that
            # had some white-spaces around them. Therefore: strip()
            ## As of version 0.7.8 we are supporting white space
            ## separated values as well
            if len(row) == 1:
                row = row[0].split()
            data.append((np.float(row[0].strip())*timefactor, 
                         np.float(row[1].strip())))
            if len(row) == 5:
                # this has to be correlation with weights
                weights.append(np.float(row[4].strip()))
                if weightname == "external":
                    try:
                        weightname = "ext. "+prev_row[0].split("Weights")[1].split("[")[1].split("]")[0]
                    except:
                        pass
        prev_row = row
    # Collect the rest of the trace, if there is any:
    rest = np.array(data)
    if numtraces == 0:
        corr = rest
    elif numtraces >= 1:
        trace = rest
    del data
    ## Remove any NaN numbers from thearray
    # Explanation:
    # np.isnan(data)
    #  finds the position of NaNs in the array (True positions); 2D array, bool
    # any(1)
    #  finds the rows that have True in them; 1D array, bool
    # ~
    #  negates them and is given as an argument (array type bool) to
    #  select which items we want.
    corr = corr[~np.isnan(corr).any(1)]
    # Also check for infinities.
    corr = corr[~np.isinf(corr).any(1)]
    csvfile.close()
    Traces=list()
    # Set correct trace data for import
    if numtraces == 1 and DataType[:2] == "AC":
        Traces.append(trace)
    elif numtraces == 2 and DataType[:2] == "CC":
        Traces.append([traceA, trace])
    elif numtraces == 1 and DataType[:2] == "CC":
        # Should not happen, but for convenience:
        Traces.append([trace, trace])
    else:
        Traces.append(None)
    dictionary = dict()
    dictionary["Correlation"] = [corr]
    dictionary["Trace"] = Traces
    dictionary["Type"] = [DataType]
    dictionary["Filename"] = [filename]
    if len(weights) != 0:
        dictionary["Weight"] = [ np.array(weights)]
        dictionary["Weight Name"] = [weightname]
    return dictionary



# -*- coding: utf-8 -*-
""" 
This file contains a TIR-FCS kineteics model function according to:
"Total Internal Reflection Fluorescence Correlation Spectroscopy: Effects
of Lateral Diffusion and Surface-Generated Fluorescence"
Jonas Ries, Eugene P. Petrov, and Petra Schwille
Biophysical Journal, Volume 95, July 2008, 390–399
"""
import numpy as np                  # NumPy
import scipy.special as sps
import numpy.lib.scimath as nps

def wixi(x):
    """ Complex Error Function (Faddeeva/Voigt).
        w(i*x) = exp(x**2) * ( 1-erf(x) )
        This function is called by other functions within this module.
        We are using the scipy.special.wofz module which calculates
        w(z) = exp(-z**2) * ( 1-erf(-iz) )
        z = i*x
    """
    z = x*1j
    wixi = sps.wofz(z)
    # We should have a real solution. Make sure nobody complains about
    # some zero-value imaginary numbers.
    return np.real_if_close(wixi)


# Lateral correlation function
def CF_gxy_square(parms, tau):
    u""" 2D free diffusion measured with a square pinhole.
        For the square pinhole, the correlation function can readily be
        calculated for a TIR-FCS setup.
        This function is called by other functions within this module.
        Attention:
        This is NOT g2D (or gCC), the non normalized correlation function.
        g2D = gxy * eta**2 * Conc,
        where eta is the molecular brightness, Conc the concentration and
        gxy is this function.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D      Diffusion coefficient
        [1] sigma  lateral size of the point spread function
                   sigma = simga_0 * lambda / NA
        [2] a      side size of the square pinhole
        *tau* - lag time

        Returns: Nonnormalized Lateral correlation function w/square pinhole.
    """
    D = parms[0]
    sigma = parms[1]
    a = parms[2]

    var1 = sigma**2+D*tau
    AA = 2*np.sqrt(var1)/(a**2*np.sqrt(np.pi))
    BB = np.exp(-a**2/(4*(var1))) - 1
    CC = sps.erf(a/(2*np.sqrt(var1)))/a
    # gx = AA*BB+CC
    # gxy = gx**2
    return (AA*BB+CC)**2


def CF_gz_CC(parms, tau, wixi=wixi):
    u""" Axial (1D) diffusion in a TIR-FCS setup.
        From Two species (bound/unbound) this is the bound part.
        This function is called by other functions within this module.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D_3D     3D Diffusion coefficient (species A)
        [1] D_2D     2D Diffusion coefficient of bound species C
        [2] sigma    lateral size of the point spread function
                     sigma = simga_0 * lambda / NA
        [3] a        side size of the square pinhole
        [4] d_eva    evanescent decay length (decay to 1/e)
        [5] Conc_3D  3-dimensional concentration of species A
        [6] Conc_2D  2-dimensional concentration of species C
        [7] eta_3D   molecular brightness of species A
        [8] eta_2D   molecular brightness of species C
        [9] k_a      Surface association rate constant
        [10] k_d     Surface dissociation rate constant
        *tau* - lag time
    """
    D = parms[0]
    #D_2D = parms[1]
    #sigma = parms[2]
    # a = parms[3]
    #d_eva = parms[4]
    Conc_3D = parms[5]      # ligand concentration in solution
    Conc_2D = parms[6]
    eta_3D = parms[7]
    eta_2D = parms[8]
    k_a = parms[9]
    k_d = parms[10]
    # Define some other constants:
    K = k_a/k_d              # equilibrium constant
    Beta = 1/(1 + K*Conc_3D) # This is wrong in the Ries paper
    #Re = D / d_eva**2
    Rt = D * (Conc_3D / (Beta * Conc_2D))**2
    Rr = k_a * Conc_3D + k_d
    # Define even more constants:
    sqrtR1 = -Rr/(2*nps.sqrt(Rt)) + nps.sqrt( Rr**2/(4*Rt) - Rr )
    sqrtR2 = -Rr/(2*nps.sqrt(Rt)) - nps.sqrt( Rr**2/(4*Rt) - Rr )
    R1 = sqrtR1 **2
    R2 = sqrtR2 **2
    # Calculate return function
    A1 = eta_2D * Conc_2D / (eta_3D * Conc_3D) * Beta
    A2 = sqrtR1 * wixi(-nps.sqrt(tau*R2)) - sqrtR2 * wixi(-nps.sqrt(tau*R1))
    A3 = sqrtR1 - sqrtR2
    Sol = A1 * A2 / A3
    # There are some below numerical errors-imaginary numbers.
    # We do not want them.
    return np.real_if_close(Sol)


def CF_gz_AC(parms, tau, wixi=wixi):
    u""" Axial (1D) diffusion in a TIR-FCS setup.
        From Two species (bound/unbound) this is the cross correlation part.
        This function is called by other functions within this module.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D_3D     3D Diffusion coefficient (species A)
        [1] D_2D     2D Diffusion coefficient of bound species C
        [2] sigma    lateral size of the point spread function
                     sigma = simga_0 * lambda / NA
        [3] a        side size of the square pinhole
        [4] d_eva    evanescent decay length (decay to 1/e)
        [5] Conc_3D  3-dimensional concentration of species A
        [6] Conc_2D  2-dimensional concentration of species C
        [7] eta_3D   molecular brightness of species A
        [8] eta_2D   molecular brightness of species C
        [9] k_a      Surface association rate constant
        [10] k_d     Surface dissociation rate constant
        *tau* - lag time
    """
    D = parms[0]
    #D_2D = parms[1]
    #sigma = parms[2]
    # a = parms[3]
    d_eva = parms[4]
    Conc_3D = parms[5]      # ligand concentration in solution
    Conc_2D = parms[6]
    eta_3D = parms[7]
    eta_2D = parms[8]
    k_a = parms[9]
    k_d = parms[10]
    # Define some other constants:
    K = k_a/k_d             # equilibrium constant
    Beta = 1/(1 + K*Conc_3D)
    Re = D / d_eva**2
    Rt = D * (Conc_3D / (Beta * Conc_2D))**2
    Rr = k_a * Conc_3D + k_d
    # Define even more constants:
    sqrtR1 = -Rr/(2*nps.sqrt(Rt)) + nps.sqrt( Rr**2/(4*Rt) - Rr )
    sqrtR2 = -Rr/(2*nps.sqrt(Rt)) - nps.sqrt( Rr**2/(4*Rt) - Rr )
    R1 = sqrtR1 **2
    R2 = sqrtR2 **2
    # And even more more:
    sqrtR3 = sqrtR1 + nps.sqrt(Re)
    sqrtR4 = sqrtR2 + nps.sqrt(Re)
    #R3 = sqrtR3 **2
    #R4 = sqrtR4 **2
    # Calculate return function
    A1 = eta_2D * Conc_2D * k_d / (eta_3D * Conc_3D)
    A2 = sqrtR4*wixi(-nps.sqrt(tau*R1)) - sqrtR3*wixi(-nps.sqrt(tau*R2))
    A3 = ( sqrtR1 - sqrtR2 ) * wixi( nps.sqrt(tau*Re) )
    A4 = ( sqrtR1 - sqrtR2 ) * sqrtR3 * sqrtR4
    Solution = A1 * ( A2 + A3 ) / A4
    # There are some below numerical errors-imaginary numbers.
    # We do not want them.
    return np.real_if_close(Solution)


def CF_gz_AA(parms, tau, wixi=wixi):
    u""" Axial (1D) diffusion in a TIR-FCS setup.
        From Two species (bound/unbound) this is the unbound part.
        This function is called by other functions within this module.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D_3D     3D Diffusion coefficient (species A)
        [1] D_2D     2D Diffusion coefficient of bound species C
        [2] sigma    lateral size of the point spread function
                     sigma = simga_0 * lambda / NA
        [3] a        side size of the square pinhole
        [4] d_eva    evanescent decay length (decay to 1/e)
        [5] Conc_3D  3-dimensional concentration of species A
        [6] Conc_2D  2-dimensional concentration of species C
        [7] eta_3D   molecular brightness of species A
        [8] eta_2D   molecular brightness of species C
        [9] k_a      Surface association rate constant
        [10] k_d     Surface dissociation rate constant
        *tau* - lag time
    """
    D = parms[0]
    #D_2D = parms[1]
    #sigma = parms[2]
    # a = parms[3]
    d_eva = parms[4]
    Conc_3D = parms[5]      # ligand concentration in solution
    Conc_2D = parms[6]
    eta_3D = parms[7]
    eta_2D = parms[8]
    k_a = parms[9]
    k_d = parms[10]
    # Define some other constants:
    d = d_eva
    K = k_a/k_d             # equilibrium constant
    Beta = 1/(1 + K*Conc_3D)
    Re = D / d_eva**2
    Rt = D * (Conc_3D / (Beta * Conc_2D))**2
    Rr = k_a * Conc_3D + k_d
    # Define even more constants:
    sqrtR1 = -Rr/(2*nps.sqrt(Rt)) + nps.sqrt( Rr**2/(4*Rt) - Rr )
    sqrtR2 = -Rr/(2*nps.sqrt(Rt)) - nps.sqrt( Rr**2/(4*Rt) - Rr )
    R1 = sqrtR1 **2
    R2 = sqrtR2 **2
    # And even more more:
    sqrtR3 = sqrtR1 + nps.sqrt(Re)
    sqrtR4 = sqrtR2 + nps.sqrt(Re)
    R3 = sqrtR3 **2
    R4 = sqrtR4 **2
    # Calculate return function
    Sum1 = d * nps.sqrt( Re*tau/np.pi )
    Sum2 = -d/2*(2*tau*Re -1) * wixi(np.sqrt(tau*Re))
    Sum3Mult1 = - eta_2D * Conc_2D * k_d / ( eta_3D * Conc_3D * 
                                            (sqrtR1 - sqrtR2) )
    S3M2S1M1 = sqrtR1/R3
    S3M2S1M2S1 = wixi(-nps.sqrt(tau*R1)) + -2*nps.sqrt(tau*R3/np.pi)
    S3M2S1M2S2 = ( 2*tau*sqrtR1*nps.sqrt(Re) + 2*tau*Re -1 ) * \
                 wixi(nps.sqrt(tau*Re))
    S3M2S2M1 = -sqrtR2/R4
    S3M2S2M2S1 = wixi(-nps.sqrt(tau*R2)) + -2*nps.sqrt(tau*R4/np.pi)
    S3M2S2M2S2 = ( 2*tau*sqrtR2*nps.sqrt(Re) + 2*tau*Re -1 ) * \
                 wixi(nps.sqrt(tau*Re))
    Sum3 = Sum3Mult1 * ( S3M2S1M1 * (S3M2S1M2S1 + S3M2S1M2S2) + 
                         S3M2S2M1 * (S3M2S2M2S1 + S3M2S2M2S2) )
    Sum = Sum1 + Sum2 + Sum3
    # There are some below numerical errors-imaginary numbers.
    # We do not want them.
    return np.real_if_close(Sum)


# 3D-2D binding/unbinding TIRF
def CF_Gxyz_TIR_square_ubibi(parms, tau, 
                         gAAz=CF_gz_AA, gACz=CF_gz_AC, gCCz=CF_gz_CC,
                         gxy=CF_gxy_square):
    u""" Two-component two- and three-dimensional diffusion
        with a square-shaped lateral detection area taking into account
        the size of the point spread function; and an exponential
        decaying profile in axial direction. This model covers binding
        and unbinding kintetics. 

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D_3D    Diffusion coefficient of freely diffusing species A
        [1] D_2D    Diffusion coefficient of bound species C
        [2] σ       Lateral size of the point spread function
                    σ = σ₀ * λ / NA
        [3] a       Side size of the square-shaped detection area 
        [4] d_eva   Evanescent decay constant
        [5] C_3D    Concentration of species A in observation volume
        [6] C_2D    Concentration of species C in observation area
        [7] η_3D    Molecular brightness of species A
        [8] η_2D    Molecular brightness of species C
        [9] k_a     Surface association rate constant
        [10] k_d    Surface dissociation rate constant
        *tau* - lag time

        Returns: 3D correlation function for TIR-FCS w/square pinhole and
                 surface binding/unbinding.

        Model introduced in:
         Jonas Ries, Eugene P. Petrov, and Petra Schwille
         Total Internal Reflection Fluorescence Correlation Spectroscopy: 
         Effects of Lateral Diffusion and Surface-Generated Fluorescence
         Biophysical Journal, Vol.95, July 2008, 390–399
    """
    D_3D = parms[0]
    D_2D = parms[1]
    sigma = parms[2]
    a = parms[3]
    kappa = 1/parms[4]
    Conc_3D = parms[5]
    Conc_2D = parms[6]
    eta_3D = parms[7]
    eta_2D = parms[8]
    #k_a = parms[9]
    #k_d = parms[10]
    ## We now need to copmute a real beast:
    # Inter species non-normalized correlation functions
    # gAA = gAAz * gxy(D_3D)
    # gAC = gACz * np.sqrt ( gxy(D_3D) * gxy(D_2D) )
    # gCC = gCCz * gxy(D_2D)
    # Nonnormalized correlation function
    # g = eta_3D * Conc_3D * ( gAA + 2*gAC + gCC )
    # Expectation value of fluorescence signal
    # F = eta_3D * Conc_3D / kappa + eta_2D * Conc_2D
    # Normalized correlation function
    # G = g / F**2
    ##
    # Inter species non-normalized correlation functions
    # The gijz functions take the same parameters as this function
    # The gxy function needs two different sets of parameters, depending
    # on the diffusion constant used.
    #    [0] D: Diffusion coefficient
    #    [1] sigma: lateral size of the point spread function
    #    [3] a: side size of the square pinhole
    parms_xy_2D = [D_2D, sigma, a]
    parms_xy_3D = [D_3D, sigma, a]
    # Here we go.
    gAA = gAAz(parms, tau) * gxy(parms_xy_3D, tau)
    gAC = gACz(parms, tau) * nps.sqrt( gxy(parms_xy_3D, tau) *
                                       gxy(parms_xy_2D, tau) )
    gCC = gCCz(parms, tau) * gxy(parms_xy_2D, tau)
    # Nonnormalized correlation function
    g = eta_3D * Conc_3D * ( gAA + 2*gAC + gCC )
    # Expectation value of fluorescence signal
    F = eta_3D * Conc_3D / kappa + eta_2D * Conc_2D
    # Normalized correlation function
    G = g / F**2
    # There are some below numerical errors-imaginary numbers.
    # We do not want them.
    return G.real
    #FNEW = eta_3D * Conc_3D / kappa
    #GNEW = eta_3D * Conc_3D * gCCz(parms, tau) / FNEW**2
    #return GNEW.real





# 3D-2D binding Model TIR
m_tir_3d_2d_ubib6021 = [6021, u"3D+2D+kin",
                        "Surface binding and unbinding, 3D TIR",
                        CF_Gxyz_TIR_square_ubibi]
labels_6021 = [u"D_3D [10 µm²/s]",
               u"D_2D [10 µm²/s]",
               u"σ [100 nm]",
               u"a [100 nm]", 
               u"d_eva [100 nm]", 
               u"C_3D [1000 /µm³]", 
               u"C_2D[100 /µm²]", 
               u"η_3D", 
               u"η_2D", 
               u"k_a [µm³/s]", 
               u"k_d [10³ /s]"
                ]
values_6021 = [
                9.0,      # D_3D [10 µm²/s]
                0.0,      # D_2D [10 µm²/s]
                2.3,      # σ [100 nm]
                7.50,     # a [100 nm]
                1.0,      # d_eva [100 nm]
                0.01,     # conc.3D [1000 /µm³]
                0.03,     # conc.2D [100 /µm²]
                1,        # η_3D
                1,        # η_2D
                0.00001,  # k_a [µm³/s]
                0.000064  # k_d [10³ /s]
                ]        
valuestofit_6021 = [False, True, False, False, False, False, True, False,
                    False, False, False]
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6021 = [
                u"D_3D [µm²/s]",
                u"D_2D [µm²/s]",
                u"σ [nm]",
                u"a [nm]", 
                u"d_eva [nm]", 
                u"C_3D [1/µm³]", 
                u"C_2D [1/µm²]", 
                u"η_3D", 
                u"η_2D", 
                u"k_a [µm³/s]", 
                u"k_d [1/s]"
                ]
values_factor_human_readable_6021 = [10, # "D_3D [µm²/s]",
                10,     # D_2D [10 µm²/s]
                100,    # σ [100 nm]
                100,    # a [100 nm]
                100,    # d_eva [100 nm]
                1000,   # conc.3D [1000 /µm³]
                100,    # conc.2D [100 /µm²]
                1,      # η_3D
                1,      # η_2D
                1,      # k_a [µm³/s]
                1000   # k_d [10³ /s]
                ]
parms_6021 = [labels_6021, values_6021, valuestofit_6021,
              labels_human_readable_6021, values_factor_human_readable_6021]


model1 = dict()
model1["Parameters"] = parms_6021
model1["Definitions"] = m_tir_3d_2d_ubib6021
model1["Verification"] = lambda parms: np.abs(parms)


Modelarray = [model1]



# -*- coding: utf-8 -*-
"""
PyCorrFit

Module plotting
Everything about plotting with matplotlib is located here.
Be sure to install texlive-science and texlive-math-extra
"""


import codecs
import numpy as np
import matplotlib
# We do catch warnings about performing this before matplotlib.backends stuff
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    matplotlib.use('WXAgg') # Tells matplotlib to use WxWidgets for dialogs
import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt
# Text rendering with matplotlib
from matplotlib import rcParams
import unicodedata

# For finding latex tools
from .misc import findprogram
from . import models as mdls


def greek2tex(char):
    """ Converts greek UTF-8 letters to latex """
    #decchar = codecs.decode(char, "UTF-8")
    decchar = char
    repres = unicodedata.name(decchar).split(" ")
    # GREEK SMALL LETTER ALPHA
    if repres[0] == "GREEK" and len(repres) == 4:
        letter = repres[3].lower()
        if repres[1] != "SMALL":
            letter = letter[0].capitalize() + letter[1:]
        return "\\"+letter
    else:
        return char


def escapechars(string):
    """ For latex output, some characters have to be escaped with a "\\" """
    #string = codecs.decode(string, "UTF-8")
    escapechars = ["#", "$", "%", "&", "~", "_", "\\", "{", "}"] 
    retstr = ur""
    for char in string:
        if char in escapechars:
            retstr += "\\"
            retstr += char
        elif char == "^":
            # Make a hat in latex without $$?
            retstr += "$\widehat{~}$"
        else:
            retstr += char
    return retstr


def latexmath(string):
    """ Format given parameters to nice latex. """
    if string == "offset":
        # prohibit the very often "offset" to be displayed as variables
        return r"\mathrm{offset}"
    elif string == "SP":
        return r"\mathrm{SP}"
    #string = codecs.decode(string, "UTF-8")
    unicodechars = dict()
    #unicodechars[codecs.decode("τ", "UTF-8")] = r"\tau"
    #unicodechars[codecs.decode("µ", "UTF-8")] = r"\mu"
    unicodechars[codecs.decode("²", "UTF-8")] = r"^2"
    unicodechars[codecs.decode("³", "UTF-8")] = r"^3"
    unicodechars[codecs.decode("₁", "UTF-8")] = r"_1"
    unicodechars[codecs.decode("₂", "UTF-8")] = r"_2"
    unicodechars[codecs.decode("₀", "UTF-8")] = r"_0"
    #unicodechars[codecs.decode("α", "UTF-8")] = r"\alpha"
    # We need lambda in here, because unicode names it lamda sometimes.
    unicodechars[codecs.decode("λ", "UTF-8")] = r"\lambda"
    #unicodechars[codecs.decode("η", "UTF-8")] = r'\eta'
    unitchars = dict()
    unitchars[codecs.decode("µ", "UTF-8")] = r"\micro "
    items = string.split(" ", 1)
    a = items[0]
    if len(items) > 1:
        b = items[1]
        if b.count(u"µ"):
            # Use siunitx with the upright µ
            bnew = ur"[\SI{}{"
            for char in b.strip("[]"):
                if char in unitchars.keys():
                    bnew += unitchars[char]
                else:
                    bnew += char
            b = bnew+ur"}]"
    else:
        b = ""
    anew = ur""
    for char in a:
        if char in unicodechars.keys():
            anew += unicodechars[char]
        elif char != greek2tex(char):
            anew += greek2tex(char)
        else:
            anew += char
    # lower case
    lcitems = anew.split("_",1)
    if len(lcitems) > 1:
        anew = lcitems[0]+"_{\\text{"+lcitems[1]+"}}"
    return anew + r" \hspace{0.3em} \mathrm{"+b+r"}"


def savePlotCorrelation(parent, dirname, Page, uselatex=False,
                        verbose=False, show_weights=True):
    """ Save plot from Page into file        
        Parameters:
        *parent*    the parent window
        *dirname*   directory to set on saving
        *Page*      Page containing all variables
        *uselatex*  Whether to use latex for the ploting or not.
        This function uses a hack in misc.py to change the function
        for saving the final figure. We wanted save in the same directory
        as PyCorrFit was working and the filename should be the tabtitle.
    """
    # Close all other plots before commencing
    try:
        plt.close()
    except:
        pass
    # get data
    corr = Page.corr
    dataexp = corr.correlation_plot
    resid = corr.residuals_plot
    fit = corr.modeled_plot

    weights = Page.weights_plot_fill_area
    tabtitle = Page.tabtitle.GetValue()
    #fitlabel = ur"Fit model: "+str(mdls.modeldict[Page.modelid][0])
    fitlabel = Page.corr.fit_model.name
    labelweights = ur"Weights of fit"
    labels, parms = mdls.GetHumanReadableParms(Page.modelid,
                                               corr.fit_parameters)
    if dataexp is None:
        if tabtitle.strip() == "":
            fitlabel = Page.corr.fit_model.name
        else:
            fitlabel = tabtitle
    else:
        if tabtitle.strip() == "":
            tabtitle = "page"+str(Page.counter).strip().strip(":")
    if Page.corr.normparm is not None:
        fitlabel += ur", normalized to "+Page.corr.fit_model.parameters[0][Page.corr.normparm]

    ## Check if we can use latex for plotting:
    r1 = findprogram("latex")[0]
    r2 = findprogram("dvipng")[0]
    # Ghostscript
    r31 = findprogram("gs")[0]
    r32 = findprogram("mgs")[0] # from miktex
    r3 = max(r31,r32)
    if r1+r2+r3 < 3:
        uselatex = False
    if uselatex == True:
        rcParams['text.usetex']=True
        rcParams['text.latex.unicode']=True
        rcParams['font.family']='serif'
        rcParams['text.latex.preamble']=[r"""\usepackage{amsmath}
                                            \usepackage[utf8x]{inputenc}
                                            \usepackage{amssymb}
                                            \usepackage{siunitx}"""] 
        fitlabel = ur"{\normalsize "+escapechars(fitlabel)+r"}"
        tabtitle = ur"{\normalsize "+escapechars(tabtitle)+r"}"
        labelweights = ur"{\normalsize "+escapechars(labelweights)+r"}"
    else:
        rcParams['text.usetex']=False
    # create plot
    # plt.plot(x, y, '.', label = 'original data', markersize=5)
    fig=plt.figure()
    fig.canvas.set_window_title("Correlation - "+Page.title)
    if resid is not None:
        gs = gridspec.GridSpec(2, 1, height_ratios=[5,1])
        ax = plt.subplot(gs[0])
    else:
        ax = plt.subplot(111)
        #    ax = plt.axes()
    ax.semilogx()
    # plot fit first
    plt.plot(fit[:,0], fit[:,1], '-', label=fitlabel, lw=1.5,
             color="blue")
    if dataexp is not None:
        plt.plot(dataexp[:,0], dataexp[:,1], '-', color="black",
                 alpha=.7, label=tabtitle, lw=1)
    else:
        plt.xlabel(ur'lag time $\tau$ [ms]')
    
    if weights is not None and show_weights is True:
        plt.fill_between(weights[0][:,0],weights[0][:,1],weights[1][:,1],
                         color='cyan')
        # fake legend:
        p = plt.Rectangle((0, 0), 0, 0, color='cyan',
                          label=labelweights)
        ax.add_patch(p)
    plt.ylabel('correlation')
    if dataexp is not None:
        mind = np.min([ dataexp[:,1], fit[:,1]])
        maxd = np.max([ dataexp[:,1], fit[:,1]])
    else:
        mind = np.min(fit[:,1])
        maxd = np.max(fit[:,1])
    ymin = mind - (maxd - mind)/20.
    ymax = maxd + (maxd - mind)/20.
    ax.set_ylim(bottom=ymin, top=ymax)
    xmin = np.min(fit[:,0])
    xmax = np.max(fit[:,0])
    ax.set_xlim(xmin, xmax)
    # Add some nice text:
    if uselatex == True and len(parms) != 0:
        text = r""
        text += r'\['            #every line is a separate raw string...
        text += r'\begin{split}' # ...but they are all concatenated
        #                          by the interpreter :-)
        for i in np.arange(len(parms)):
            text += ur' {} &= {:.3g} \\'.format(latexmath(labels[i]), parms[i])
        ## According to issue #54, we remove fitting errors from plots
        #if errparms is not None:
        #    keys = errparms.keys()
        #    keys.sort()
        #    for key in keys:
        #        text += r' \Delta '+latexmath(key)+r" &= " + str(errparms[key]) +r' \\ '
        text += r' \end{split} '
        text += r' \] '
    else:
        text = ur""
        for i in np.arange(len(parms)):
            text += u"{} = {:.3g}\n".format(labels[i], parms[i])
        ## According to issue #54, we remove fitting errors from plots
        #if errparms is not None:
        #    keys = errparms.keys()
        #    keys.sort()
        #    for key in keys:
        #        text += "Err "+key+" = " + str(errparms[key]) +"\n"



    logmax = np.log10(xmax)
    logmin = np.log10(xmin)
    logtext = 0.6*(logmax-logmin)+logmin
    xt = 10**(logtext)
    yt = 0.3*ymax
    plt.text(xt,yt,text, size=12)
    if resid is not None:
        ax2 = plt.subplot(gs[1])
        #ax2 = plt.axes()
        ax2.semilogx()
        if Page.corr.is_weighted_fit:
            if uselatex == True:
                lb = r"\newline \indent "
            else:
                lb = "\n"
            yLabelRes = "weighted "+ lb +"residuals"
        else:
            yLabelRes = "residuals"
        minx = np.min(resid[:,0])
        maxx = np.max(resid[:,0])
        miny = np.min(resid[:,1])
        maxy = np.max(resid[:,1])
        plt.hlines(0, minx, maxx, colors="orange")
        plt.plot(resid[:,0], resid[:,1], '-', color="black",
                 alpha=.85, label=yLabelRes, lw=1)
        plt.xlabel(r'lag time $\tau$ [ms]')
        plt.ylabel(yLabelRes, multialignment='center')

        ax2.set_xlim(minx, maxx)
        maxy = max(abs(maxy), abs(miny))
        ax2.set_ylim(-maxy, maxy)
        ticks = ax2.get_yticks()
        ax2.set_yticks([ticks[0], ticks[-1], 0])
    ## Hack
    # We need this for hacking. See edclasses.
    fig.canvas.HACK_parent = parent
    fig.canvas.HACK_fig = fig
    fig.canvas.HACK_Page = Page
    fig.canvas.HACK_append = ".png"
    

    # Legend outside of plot
    # Decrease size of plot to fit legend
    box = ax.get_position()
    
    ax.set_position([box.x0, box.y0 + box.height * 0.2,
                     box.width, box.height * 0.9])
    
    if resid is not None:
        box2 = ax2.get_position()
        ax2.set_position([box2.x0, box2.y0 + box.height * 0.2,
                     box2.width, box2.height])
    
    ax.legend(loc='upper center', bbox_to_anchor=(0.5, -0.55),
              prop={'size':9})
    
    
    if verbose == True:
        plt.show()
    else:
        # If WXAgg is not used for some reason, then our hack does not work
        # and we must use e.g. TkAgg
        try:
            fig.canvas.toolbar.save()
        except AttributeError:
            fig.canvas.toolbar.save_figure()
        # Close all other plots before commencing
        try:
            plt.close()
        except:
            pass


def savePlotTrace(parent, dirname, Page, uselatex=False, verbose=False):
    """ Save trace plot from Page into file        
        Parameters:
        *parent*    the parent window
        *dirname*   directory to set on saving
        *Page*      Page containing all variables
        *uselatex*  Whether to use latex for the ploting or not.
        This function uses a hack in misc.py to change the function
        for saving the final figure. We wanted save in the same directory
        as PyCorrFit was working and the filename should be the tabtitle.
    """
    # Close all other plots before commencing
    try:
        plt.close()
    except:
        pass
    # Trace must be displayed in s
    timefactor = 1e-3
    tabtitle = Page.tabtitle.GetValue()
    if tabtitle.strip() == "":
        tabtitle = "page"+str(Page.counter).strip().strip(":")
    # Intensity trace in kHz may stay the same
    if len(Page.corr.traces) == 0:
        return
    
    traces = Page.corr.traces
    labels = list()
    for ii, tr in enumerate(traces):
        labels.append("Channel {}: {}".format(ii+1, tr.name))

    ## Check if we can use latex for plotting:
    r1 = findprogram("latex")[0]
    r2 = findprogram("dvipng")[0]
    # Ghostscript
    r31 = findprogram("gs")[0]
    r32 = findprogram("mgs")[0]
    r3 = max(r31,r32)
    if r1+r2+r3 < 3:
        uselatex = False
    if uselatex == True:
        rcParams['text.usetex']=True
        rcParams['text.latex.unicode']=True
        rcParams['font.family']='serif'
        rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"] 
        for i in np.arange(len(labels)):
            labels[i] = ur"{\normalsize "+escapechars(labels[i])+r"}"
    else:
        rcParams['text.usetex']=False
    # create plot
    # plt.plot(x, y, '.', label = 'original data', markersize=5)
    fig=plt.figure(figsize=(10,3))
    fig.canvas.set_window_title("Trace - "+Page.title)
    ax = plt.subplot(111)
    for i in np.arange(len(traces)):
        # Columns
        time = traces[i][:,0]*timefactor
        intensity = traces[i][:,1]
        plt.plot(time, intensity, '-', 
                 label = labels[i],
                 lw=1)
    # set plot boundaries
    maxy = -np.infty
    miny = np.infty
    for tr in traces:
        maxy = max(np.max(tr[:,1]), maxy)
        miny = min(np.min(tr[:,1]), miny)
    ax.set_ylim(miny, maxy)

    plt.ylabel('count rate [kHz]')
    plt.xlabel('time [s]')
    
    # Legend outside of plot
    # Decrease size of plot to fit legend
    box = ax.get_position()
    ax.set_position([box.x0, box.y0 + box.height * 0.2,
                     box.width, box.height * 0.9])
    plt.legend(loc='upper center', 
               bbox_to_anchor=(0.5, -0.35),
               prop={'size':9},
               )
    
    ## Hack
    # We need this for hacking. See edclasses.
    fig.canvas.HACK_parent = parent
    fig.canvas.HACK_fig = fig
    fig.canvas.HACK_Page = Page
    fig.canvas.HACK_append = "_trace.png"

    plt.tight_layout(rect=(.001,.34,.999,1.0))

    if verbose == True:
        plt.show()
    else:
        # If WXAgg is not used for some reason, then our hack does not work
        # and we must use e.g. TkAgg
        try:
            fig.canvas.toolbar.save()
        except AttributeError:
            fig.canvas.toolbar.save_figure()
        # Close all other plots before commencing
        try:
            plt.close()
        except:
            pass

# set dpi to 300
matplotlib.rcParams['savefig.dpi'] = 300